@@ -30,6 +30,8 @@
 
 #include "ngsfixture.hpp"
 
+#include <os-native.h>
+
 #include <ngs/Statistics.hpp>
 
 #include <klib/text.h>
@@ -527,7 +529,7 @@ const char UsageDefaultName[] = "test-ngs_sradb-c++";
 
 rc_t CC KMain ( int argc, char *argv [] )
 {
-    putenv("NCBI_VDB_QUALITY=R");
+    setenv("NCBI_VDB_QUALITY", "R", 1);
     KConfigDisableUserSettings();
     rc_t rc=NgsSradbCppTestSuite(argc, argv);
     NgsFixture::ReleaseCache();
