@@ -23,6 +23,10 @@ env NCBI_SETTINGS="${SETTINGS_FILE}" \
     SRATOOLS_IMPERSONATE=fastq-dump \
     "${SRATOOLS}" --ngc foo.ngc --perm foo.jwt --cart foo.cart DRX000001 2>actual/testing.stderr
 
+env NCBI_SETTINGS="${SETTINGS_FILE}" \
+    SRATOOLS_IMPERSONATE=vdb-dump \
+    "${SRATOOLS}" --ngc CMakeLists.txt foobar 2>actual/testing2.stderr
+
 rm -rf "${SETTINGS_FILE}"
 
 diff -q expected/testing.stderr actual/testing.stderr >/dev/null || \
@@ -33,4 +37,11 @@ diff -q expected/testing-cloudy.stderr actual/testing.stderr >/dev/null || \
     diff expected/testing-cloudy.stderr actual/testing.stderr
     exit 1
 }
+
+diff -q expected/testing2.stderr actual/testing2.stderr >/dev/null || \
+{
+    echo "Driver tool test 'testing' FAILED!"
+    diff expected/testing2.stderr actual/testing2.stderr
+    exit 1
+}
 echo "Driver tool test 'testing' via $2 finished."
