@@ -1,4 +1,4 @@
-#!/bin/bash
+# bash
 
 bin_dir=$1
 srapath=$2
@@ -38,10 +38,10 @@ if [ "$res" == "0" ];
 	then echo "error test FAILED, res=$res output=$output" && exit 1;
 fi
 
-grep -q "ncbi_phid=" <<< "$output" > /dev/null
+echo "${output}" | grep -q "ncbi_phid=" > /dev/null
 res=$?
 if [ "$res" != "0" ];
 	then echo "error test FAILED (no "ncbi_phid" found), res=$res output=$output" && exit 1;
 fi
 
-echo error test is finished
\ No newline at end of file
+echo error test is finished
