@@ -2,16 +2,16 @@ $acc = 'SRR8639211';
 `rm -fr $acc` ; die if $?;
 
 # 1. do not fail if directory does not exist
-`../../scripts/fix-sra-names.sh $acc` ; die if $?;
+`bash ../../scripts/fix-sra-names.sh $acc` ; die if $?;
 
 # 2. single run file
 `mkdir $acc` ; die if $?;
 `touch $acc/$acc.1` ; die if $?;
 
 # 3. fail if run without arguments
-`../../scripts/fix-sra-names.sh` ; die unless $?;
+`bash ../../scripts/fix-sra-names.sh` ; die unless $?;
 
-`../../scripts/fix-sra-names.sh $acc` ; die if $?;
+`bash ../../scripts/fix-sra-names.sh $acc` ; die if $?;
 @d = `ls $acc | sort`;
 die if $#d != 0;
 die if $d[0] ne "$acc.sra\n";
@@ -21,7 +21,7 @@ die if $d[0] ne "$acc.sra\n";
 `mkdir $acc` ; die if $?;
 `touch $acc/$acc.1` ; die if $?;
 `touch $acc/$acc.vdbcache.1` ; die if $?;
-`../../scripts/fix-sra-names.sh $acc` ; die if $?;
+`bash ../../scripts/fix-sra-names.sh $acc` ; die if $?;
 @d = `ls $acc | sort`;
 die if $#d != 1;
 die if $d[0] ne "$acc.sra\n";
@@ -32,7 +32,7 @@ die if $d[1] ne "$acc.sra.vdbcache\n";
 `mkdir $acc` ; die if $?;
 `touch $acc/$acc.sra` ; die if $?;
 `touch $acc/$acc.vdbcache.1` ; die if $?;
-`../../scripts/fix-sra-names.sh $acc` ; die if $?;
+`bash ../../scripts/fix-sra-names.sh $acc` ; die if $?;
 @d = `ls $acc | sort`;
 die if $#d != 1;
 die if $d[0] ne "$acc.sra\n";
@@ -43,7 +43,7 @@ die if $d[1] ne "$acc.vdbcache.1\n";
 `mkdir $acc` ; die if $?;
 `touch $acc/$acc.1` ; die if $?;
 `touch $acc/$acc.sra.vdbcache` ; die if $?;
-`../../scripts/fix-sra-names.sh $acc` ; die if $?;
+`bash ../../scripts/fix-sra-names.sh $acc` ; die if $?;
 @d = `ls $acc | sort`;
 die if $#d != 1;
 die if $d[0] ne "$acc.1\n";
