@@ -94,7 +94,10 @@ std::vector<Response2::ResultEntry::Flat> Response2::ResultEntry::flatten() cons
 Response2::ResultEntry::TypeIndex Response2::ResultEntry::indexTypes() const
 {
     auto result = TypeIndex();
-    for (int i = 0; i < (int)files.size(); ++i) {
+    for (auto const &file : files) {
+        result[file.type] = TypeIndex::value_type::second_type();
+    }
+    for (int i = 0; i < (int)flattened.size(); ++i) {
         auto const &p = flattened[i];
         auto const &file = files[p.first];
         result[file.type].push_back(i);
@@ -121,34 +124,38 @@ int Response2::ResultEntry::getCacheFor(Flattened const &match) const
     auto const isFromNCBI = (service == "ncbi" || service == "sra-ncbi");
     auto const encrypted = isFromNCBI ? match.second.projectId.has_value() : false;
     auto const fnd = byType.find("vdbcache");
-    if (fnd != byType.end()) {
-        for (auto i : fnd->second) {
-            auto const &p = flattened[i];
-            auto const &file = files[p.first];
-            auto const &location = file.locations[p.second];
+    if (fnd == byType.end())
+        return -1;
+    for (auto i : fnd->second) {
+        auto const &p = flattened[i];
+        auto const &file = files[p.first];
 
-            if (location.service == service && location.region == region)
+        if (file.noqual != match.first.noqual) ///< quality must match
+            continue;
+
+        auto const &location = file.locations[p.second];
+
+        if (location.service == service && location.region == region)
+            return i; ///< a solid match, look no further
+
+        if (location.service == "ncbi" || location.service == "sra-ncbi") {
+            auto const thisEncrypted = location.projectId.has_value();
+            if (thisEncrypted && encrypted && match.second.projectId == location.projectId)
                 return i; ///< a solid match, look no further
-
-            if (location.service == "ncbi" || location.service == "sra-ncbi") {
-                auto const thisEncrypted = location.projectId.has_value();
-                if (thisEncrypted && encrypted && match.second.projectId == location.projectId)
-                    return i; ///< a solid match, look no further
-                else if (!thisEncrypted)
-                    result = i; ///< a weak match, maybe there's a better one
-            }
+            else if (!thisEncrypted)
+                result = i; ///< a weak match, maybe there's a better one
         }
     }
     return result;
 }
 
+using Response2Data = Response2;
 namespace impl {
-/// @brief holds SDL version 2 response
-/// @Note member names generally match the corresponding member names in the SDL response JSON
-struct Response2 : public ::Response2 {
-    using Base = ::Response2;
-    struct DecodingError : public ::Response2::DecodingError {
-        using Base = ::Response2::DecodingError;
+/// @brief parses SDL version 2 response
+struct Response2 : public Response2Data {
+    using Base = Response2Data;
+    struct DecodingError : public Response2Data::DecodingError {
+        using Base = Response2Data::DecodingError;
 
         DecodingError(Base const &other)
         : Base(other)
@@ -224,14 +231,14 @@ struct Response2 : public ::Response2 {
         }
     };
 
-    struct ResultEntry : public ::Response2::ResultEntry {
-        using Base = ::Response2::ResultEntry;
+    struct ResultEntry : public Response2Data::ResultEntry {
+        using Base = Response2Data::ResultEntry;
 
-        struct FileEntry : public ::Response2::ResultEntry::FileEntry {
-            using Base = ::Response2::ResultEntry::FileEntry;
+        struct FileEntry : public Response2Data::ResultEntry::FileEntry {
+            using Base = Response2Data::ResultEntry::FileEntry;
 
-            struct LocationEntry : public ::Response2::ResultEntry::FileEntry::LocationEntry {
-                using Base = ::Response2::ResultEntry::FileEntry::LocationEntry;
+            struct LocationEntry : public Response2Data::ResultEntry::FileEntry::LocationEntry {
+                using Base = Response2Data::ResultEntry::FileEntry::LocationEntry;
 
                 struct Delegate final : public JSONParser::Delegate {
                     JSONValueDelegate<std::string> link, service, region, expirationDate, projectId;
@@ -322,7 +329,7 @@ struct Response2 : public ::Response2 {
                             DecodingError::no_default, DecodingError::locations
                         }).setVictim("region").setCause("service", service).setLocation(service);
                     }
-                    static std::ostream &print(std::ostream &os, ::Response2::DecodingError const &err) {
+                    static std::ostream &print(std::ostream &os, Response2Data::DecodingError const &err) {
                         assert(err.object == Response2::DecodingError::locations);
                         switch (err.errorType) {
                         case Response2::DecodingError::missing:
@@ -462,7 +469,7 @@ struct Response2 : public ::Response2 {
                     }).setVictim("type").setCause("object", object);
                 }
 
-                static std::ostream &print(std::ostream &os, ::Response2::DecodingError const &err) {
+                static std::ostream &print(std::ostream &os, Response2Data::DecodingError const &err) {
                     assert(err.object == Response2::DecodingError::files);
                     switch (err.errorType) {
                     case Response2::DecodingError::missing:
@@ -557,7 +564,7 @@ struct Response2 : public ::Response2 {
                     , files.get()
                 };
             }
-            static std::ostream &print(std::ostream &os, ::Response2::DecodingError const &err) {
+            static std::ostream &print(std::ostream &os, Response2Data::DecodingError const &err) {
                 assert(err.object == Response2::DecodingError::result);
                 switch (err.errorType) {
                 case Response2::DecodingError::missing:
@@ -611,7 +618,7 @@ struct Response2 : public ::Response2 {
 
         Base get()
         {
-            auto error = DecodingError(DecodingError::Base {
+            auto error = DecodingError(DecodingError::Base{
                 DecodingError::missing, DecodingError::topLevel
             });
             if (!wasSet)
@@ -643,7 +650,7 @@ struct Response2 : public ::Response2 {
                 , message ? message.get() : ""
             };
         }
-        static std::ostream &print(std::ostream &os, ::Response2::DecodingError const &err) {
+        static std::ostream &print(std::ostream &os, Response2Data::DecodingError const &err) {
             switch (err.errorType) {
             case Response2::DecodingError::missing:
                 if (!err.haveVictim())
