@@ -237,7 +237,7 @@ void data_sources::set_ce_token_env_var() const {
     SETENV_IF(have_ce_token, CE_TOKEN, ce_token_);
 }
 
-void data_sources::set_param_bits_env_var(uint64_t bits) const {
+void data_sources::set_param_bits_env_var(uint64_t bits) {
     char buffer[32];
     auto i = sizeof(buffer);
 
@@ -296,31 +296,25 @@ data_sources::QualityPreference data_sources::qualityPreference() {
     return result;
 }
 
-static bool setIfExists(Dictionary &result, std::string const &key, std::string const &name)
+static bool setIfExists(Dictionary &result, std::string const &key, FilePath const &cwd, std::string const &name)
 {
-    if (FilePath(name.c_str()).exists()) {
-        result[key] = name;
-        return true;
-    }
-    return false;
-}
-
-static bool cwdSetIfExists(Dictionary &result, std::string const &key, FilePath const &cwd, std::string const &name)
-{
-    if (FilePath::exists(name)) {
-        result[key].assign(cwd.append(name));
+    auto const full = cwd.append(name);
+    if (full.exists()) {
+        LOG(9) << name << " exists: setting '" << key << "' = '" << (std::string)full << "'." << std::endl;
+        result[key].assign(full);
         return true;
     }
+    LOG(9) << name << " does not exist." << std::endl;
     return false;
 }
 
 /// \brief Adds extensions to name and checks for existence.
 /// \note The invariant is `result["path"]` is set to the name of the file system object if it exists.
 /// \returns true if it is a file system object.
-static bool tryWithSraExtensions(Dictionary &result, std::string const &name, bool const wantFullQuality)
+static bool tryWithSraExtensions(Dictionary &result, FilePath const &cwd, std::string const &name, bool const wantFullQuality)
 {
     for (auto ext : Accession::extensionsFor(wantFullQuality)) {
-        if (setIfExists(result, "path", name + ext))
+        if (setIfExists(result, "path", cwd, name + ext))
             return true;
     }
     return false;
@@ -335,22 +329,22 @@ static void lookForCacheFileIn(FilePath const &directory, Dictionary &result, bo
     auto temp = base;
 
     // try acc.ext.vdbcache
-    if (cwdSetIfExists(result, key, directory, base + suffix))
+    if (setIfExists(result, key, directory, base + suffix))
         return;
 
     for (auto sep = temp.find_last_of("."); sep != temp.npos; sep = temp.find_last_of(".")) {
         temp = temp.substr(0, sep);
 
         // try acc.vdbcache.ext
-        if (cwdSetIfExists(result, key, directory, temp + suffix + base.substr(sep)))
+        if (setIfExists(result, key, directory, temp + suffix + base.substr(sep)))
             return;
 
         // try acc.vdbcache
-        if (cwdSetIfExists(result, key, directory, temp + suffix))
+        if (setIfExists(result, key, directory, temp + suffix))
             return;
     }
     // try accession.vdbcache
-    cwdSetIfExists(result, key, directory, accession.accession() + suffix);
+    setIfExists(result, key, directory, accession.accession() + suffix);
 
     (void)(wantFullQuality);
 }
@@ -359,18 +353,18 @@ static void lookForCacheFileIn(FilePath const &directory, Dictionary &result, bo
 /// \returns true if name is a file system object.
 static bool getLocalFilePath(Dictionary &result, FilePath const &cwd, std::string const &name, bool const wantFullQuality)
 {
-    if (!setIfExists(result, "path", name) && !tryWithSraExtensions(result, name, wantFullQuality))
+    if (!setIfExists(result, "path", cwd, name) && !tryWithSraExtensions(result, cwd, name, wantFullQuality))
         return false;
 
     auto const &filename = result["path"]; // might have extension added
     LOG(9) << name << " is " << filename << " in directory " << (std::string)cwd << "." << std::endl;
 
-    auto const accession = Accession(filename.c_str());
+    auto const accession = Accession(name.c_str());
     auto const exts = accession.sraExtensions();
 
     LOG(9) << filename << " has " << exts.size() << " sra extensions." << std::endl;
     if (exts.size() == 1) {
-        result[LocalKey::filePath] = result["path"];
+        result[LocalKey::filePath] = filename;
         result[LocalKey::qualityType] = exts.front().first > 0 ? Accession::qualityTypeForLite : Accession::qualityTypeForFull;
         lookForCacheFileIn(cwd, result, wantFullQuality, accession);
         return true;
@@ -411,7 +405,6 @@ std::map<std::string, Dictionary> getLocalFileInfo(CommandLine const &cmdline, A
         auto const path = cmdline.pathForArgument(arg);
         auto const dir_base = path.split();
         auto const filename = std::string(dir_base.second);
-        auto const hasDirName = !dir_base.first.empty();
 
         result[name]["name"] = name;
 
@@ -425,45 +418,37 @@ std::map<std::string, Dictionary> getLocalFileInfo(CommandLine const &cmdline, A
             return;
         LOG(9) << "Examining path: " << name << std::endl;
 
-        try {
-            path.makeCurrentDirectory();
-
-            i.second["local"] = name;
-            LOG(9) << name << " is a directory." << std::endl;
-
+        // Try to use `path` as a directory
+        if (path.makeCurrentDirectory()) {
             auto const path = FilePath::cwd(); // canonical path
             i.second["path"].assign(path);
+            i.second["local"] = name;
+            LOG(9) << name << " is a directory." << std::endl;
 
             if (isaRun && accession.extension().empty()) // accession directories don't have extensions
                 getADInfo(i.second, path, accession, wantFullQuality);
             else
                 LOG(3) << name << " is a directory, but doesn't look like an SRA Accession Directory." << std::endl;
-
             cwd.makeCurrentDirectory();
             return;
         }
-        catch (std::system_error const &e) {
-            auto const what = std::string(e.what());
-            if (what.find("chdir") == std::string::npos || what.find(name) == std::string::npos)
-                throw;
+        else {
             LOG(9) << "can't chdir to " << (std::string)path << " but that's okay."<< std::endl;
         }
 
-        if (hasDirName) {
-            LOG(9) << name << " has a directory '" << std::string(dir_base.first) << "'." << std::endl;
-            try {
-                auto const dir = dir_base.first.copy();
+        // Try to use the parent directory of `path`
+        if (!dir_base.first.empty()) {
+            auto const dir = dir_base.first;
+            LOG(9) << name << " has a directory '" << std::string(dir) << "'." << std::endl;
 
-                dir.makeCurrentDirectory();
-                {
-                    auto const path = FilePath::cwd(); // canonical path
-                    if (getLocalFilePath(i.second, path, filename, wantFullQuality))
-                        i.second["local"] = name;
-                }
+            if (dir.makeCurrentDirectory()) {
+                auto const path = FilePath::cwd(); // canonical path
+                if (getLocalFilePath(i.second, path, filename, wantFullQuality))
+                    i.second["local"] = name;
                 cwd.makeCurrentDirectory();
             }
-            catch (std::system_error const &e) {
-                LOG(1) << std::string(dir_base.first) << " is a directory, but can't chdir to it: " << e.what() << std::endl;
+            else {
+                LOG(1) << std::string(dir) << " is a directory, but can't chdir to it." << std::endl;
             }
             return;
         }
@@ -554,7 +539,7 @@ data_sources::data_sources(CommandLine const &cmdline, Arguments const &args, bo
                             if (ngc == nullptr && projectId) {
                                 std::cerr <<
                                     "The data for " << query << " from " << service << "." << region << " is encrypted.\n"
-                                    "To use this data, please get an ngc file from dbGaP for " << projectId << ", and rerun with --ngc <file>." << std::endl;
+                                    "To use this data, please get an ngc file from dbGaP for " << projectId.value() << ", and rerun with --ngc <file>." << std::endl;
                                 encrypted = true;
                                 continue;
                             }
