@@ -381,7 +381,10 @@ static bool shouldPreferLiteFormat(bool isSet, bool isFull, std::string const &t
 
 static void printHelp [[noreturn]] (bool requested = false)
 {
-    (requested ? std::cout : std::cerr) << "usage: sratools print-argbits | print-args-json | help" << std::endl;
+    (requested ? std::cout : std::cerr)
+        << "usage: sratools print-argbits | print-args-json | help\n"
+        << "Please see https://github.com/ncbi/sra-tools/wiki/sratools-driver-tool for an explanation of the sratools driver tool."
+        << std::endl;
     exit(requested ? EXIT_SUCCESS : EX_USAGE);
 }
 
@@ -420,6 +423,7 @@ static int main(CommandLine const &argv)
     enableLogging(argv.toolName.c_str());
 #endif
     LOG(7) << "executable path: " << (std::string)argv.fullPathToExe << std::endl;
+    // std::cerr << "executable path: " << (std::string)argv.fullPathToExe << std::endl;
 
     config = new Config();
     struct Defer { ~Defer() { delete config; config = nullptr; } } freeConfig;
@@ -503,8 +507,7 @@ static int main(CommandLine const &argv)
         }
 
         // MARK: include parameters-used bitfield in communications to SDL
-        // TODO: UNCOMMENT WHEN READY; SEE JIRA VDB-5001
-        // all_sources.set_param_bits_env_var(parsed.argsUsed());
+        data_sources::set_param_bits_env_var(parsed.argsUsed());
 
         // MARK: Look for tool arguments in the file system or ask SDL about them.
         auto const &all_sources = data_sources::preload(argv, parsed);
@@ -577,6 +580,16 @@ static int main(CommandLine const &argv)
 
 } // namespace sratools
 
+// BSD is defined when compiling on Mac
+// Use the MAC case below, not this one
+#if BSD && !MAC
+int main(int argc, char *argv[], char *envp[])
+{
+    auto const invocation = CommandLine(argc, argv, envp, nullptr);
+    return sratools::main(invocation);
+}
+#endif
+
 #if MAC
 int main(int argc, char *argv[], char *envp[], char *apple[])
 {
