@@ -88,12 +88,11 @@ static merge_src_t * get_min_merge_src( merge_src_t * src, uint32_t count ) {
         }
     }
     return res;
-} 
+}
 
 /* ================================================================================= */
 
 typedef struct merge_sorter_t {
-    
     struct lookup_writer_t * dst; /* lookup_writer.h */
     struct index_writer_t * idx;  /* index.h */
     merge_src_t * src;            /* vector of input-files to be merged */
@@ -112,7 +111,7 @@ static rc_t init_merge_sorter( merge_sorter_t * self,
                                struct bg_update_t * gap ) {
     rc_t rc = 0;
     uint32_t i;
-    
+
     if ( NULL != index ) {
         rc = make_index_writer( dir, &( self -> idx ), buf_size,
                         DFLT_INDEX_FREQUENCY, "%s", index ); /* index.h */
@@ -124,7 +123,7 @@ static rc_t init_merge_sorter( merge_sorter_t * self,
     self -> total_entries = 0;
     self -> num_src = num_src;
     self -> gap = gap;
-    
+
     if ( 0 == rc ) {
         rc = make_lookup_writer( dir, self -> idx, &( self -> dst ), buf_size, "%s", output ); /* lookup_writer.h */
     }
@@ -136,7 +135,7 @@ static rc_t init_merge_sorter( merge_sorter_t * self,
             ErrMsg( "init_merge_sorter2.calloc( %d ) failed", ( ( sizeof * self -> src ) * self -> num_src ) );
         }
     }
-    
+
     for ( i = 0; 0 == rc && i < self -> num_src; ++i ) {
         const char * filename;
         rc = VNameListGet ( files, i, &filename );
@@ -160,7 +159,7 @@ static void release_merge_sorter( merge_sorter_t * self ) {
     release_lookup_writer( self -> dst );
     release_index_writer( self -> idx );
     if ( NULL != self -> src ) {
-        uint32_t i;    
+        uint32_t i;
         for ( i = 0; i < self -> num_src; ++i ) {
             merge_src_t * s = &self -> src[ i ];
             release_lookup_reader( s -> reader );
@@ -177,7 +176,7 @@ static rc_t run_merge_sorter( merge_sorter_t * self ) {
     merge_src_t * to_write = get_min_merge_src( self -> src, self -> num_src ); /* above */
 
     while( 0 == rc && NULL != to_write ) {
-        rc = get_quitting();    /* helper.c */
+        rc = hlp_get_quitting();    /* helper.c */
         if ( 0 == rc ) {
             if ( last_key > to_write -> key ) {
                 rc = RC( rcVDB, rcNoTarg, rcWriting, rcFormat, rcInvalid );
@@ -196,7 +195,7 @@ static rc_t run_merge_sorter( merge_sorter_t * self ) {
                 to_write = get_min_merge_src( self -> src, self -> num_src ); /* above */
             }
             if ( 0 != rc ) {
-                set_quitting();     /* helper.c */
+                hlp_set_quitting();     /* helper.c */
             }
             bg_update_update( self -> gap, 1 ); /* signal to gap-update */
         }
@@ -225,7 +224,7 @@ typedef struct background_vector_merger_t {
     KQueue * job_q;                 /* the KVector objects arrive here from the lookup-producer */
     KThread * thread;               /* the thread that performs the merge-sort */
     struct background_file_merger_t * file_merger;    /* below */
-    struct KFastDumpCleanupTask_t * cleanup_task;     /* add the produced temp_files here too */
+    struct CleanupTask_t * cleanup_task;     /* add the produced temp_files here too */
     uint32_t product_id;            /* increased by one for each batch-run, used in temp-file-name */
     uint32_t batch_size;            /* how many KVectors have to arrive to run a batch */
     uint32_t q_wait_time;           /* timeout in milliseconds to get something out of in_q */
@@ -250,6 +249,7 @@ static void release_background_vector_merger( background_vector_merger_t * self
     if ( NULL != self -> job_q ) {
         KQueueRelease ( self -> job_q );
     }
+	KThreadRelease( self -> thread );
     free( self );
 }
 
@@ -275,7 +275,7 @@ rc_t wait_for_and_release_background_vector_merger( background_vector_merger_t *
         }
         release_background_vector_merger( self );
     }
-    
+
     if ( 0 != rc ) {
         ErrMsg( "merge_sorter.c wait_for_and_release_background_vector_merger()", rc );
     }
@@ -315,7 +315,7 @@ static bg_vec_merge_src_t * get_min_bg_vec_merge_src( bg_vec_merge_src_t * batch
         }
     }
     return res;
-} 
+}
 
 static rc_t write_bg_vec_merge_src( bg_vec_merge_src_t * src, struct lookup_writer_t * writer ) {
     rc_t rc = src -> rc;
@@ -339,7 +339,7 @@ static rc_t background_vector_merger_collect_batch( background_vector_merger_t *
     bg_vec_merge_src_t * b = calloc( self -> batch_size, sizeof * b );
     *batch = NULL;
     *count = 0;
-    
+
     if ( NULL == b ) {
         rc = RC( rcVDB, rcNoTarg, rcConstructing, rcMemory, rcExhausted );
     } else {
@@ -400,7 +400,7 @@ static rc_t background_vector_merger_process_batch( background_vector_merger_t *
         ErrMsg( "merge_sorter.c background_vector_merger_process_batch() -> %R", rc );
     } else {
         STATUS ( STAT_USR, "batch output filename is : %s", buffer );
-        rc = Add_File_to_Cleanup_Task ( self -> cleanup_task, buffer );
+        rc = clt_add_file( self -> cleanup_task, buffer );
 
         if ( 0 == rc ) {
             struct lookup_writer_t * writer; /* lookup_writer.h */
@@ -412,7 +412,7 @@ static rc_t background_vector_merger_process_batch( background_vector_merger_t *
             if ( 0 == rc ) {
                 bg_vec_merge_src_t * to_write = get_min_bg_vec_merge_src( batch, count ); /* above */
                 while( 0 == rc && NULL != to_write ) {
-                    rc = get_quitting();    /* helper.c */
+                    rc = hlp_get_quitting();    /* helper.c */
                     if ( 0 == rc ) {
                         rc = write_bg_vec_merge_src( to_write, writer ); /* above */
                         if ( 0 == rc ) {
@@ -423,14 +423,14 @@ static rc_t background_vector_merger_process_batch( background_vector_merger_t *
                         }
                         bg_update_update( self -> gap, 1 );
                         if ( 0 != rc ) {
-                            set_quitting();     /* helper.c */
+                            hlp_set_quitting();     /* helper.c */
                         }
                     }
                 }
                 release_lookup_writer( writer ); /* lookup_writer.c */
             }
         }
-        
+
         /*
         if ( rc == 0 && self -> file_merger != NULL )
             rc = lookup_check_file( self -> dir, self -> buf_size, buffer );
@@ -452,7 +452,7 @@ static rc_t CC background_vector_merger_thread_func( const KThread * thread, voi
     while( 0 == rc && !done ) {
         bg_vec_merge_src_t * batch = NULL;
         uint32_t count = 0;
-        
+
         /* Step 1 : get n = batch_size KVector's out of the in_q */
         STATUS ( STAT_USR, "collecting batch" );
         rc = background_vector_merger_collect_batch( self, &batch, &count );
@@ -505,10 +505,10 @@ rc_t make_background_vector_merger( struct background_vector_merger_t ** merger,
 
         rc = KQueueMake ( &( b -> job_q ), args -> batch_size );
         if ( 0 == rc ) {
-            rc = helper_make_thread( &( b -> thread ),
-                                     background_vector_merger_thread_func,
-                                     b,
-                                     THREAD_DFLT_STACK_SIZE );
+            rc = hlp_make_thread( &( b -> thread ),
+                                  background_vector_merger_thread_func,
+                                  b,
+                                  THREAD_DFLT_STACK_SIZE );
             if ( 0 != rc ) {
                 ErrMsg( "merge_sorter.c helper_make_thread( vector-merger ) -> %R", rc );
             }
@@ -584,7 +584,7 @@ typedef struct background_file_merger_t {
     const char * index_filename;
     locked_file_list_t files;        /* a locked file-list */
     locked_value_t sealed;           /* flag to signal if the input is sealed */
-    struct KFastDumpCleanupTask_t * cleanup_task;     /* add the produced temp_files here too */    
+    struct CleanupTask_t * cleanup_task;     /* add the produced temp_files here too */
     KThread * thread;               /* the thread that performs the merge-sort */
     uint32_t product_id;            /* increased by one for each batch-run, used in temp-file-name */
     uint32_t batch_size;            /* how many KVectors have to arrive to run a batch */
@@ -601,6 +601,7 @@ static void release_background_file_merger( background_file_merger_t * self ) {
     if ( NULL != self ) {
         locked_file_list_release( &( self -> files ), self -> dir, self -> details );
         locked_value_release( &( self -> sealed ) );
+		KThreadRelease( self -> thread );
         free( self );
     }
 }
@@ -635,11 +636,11 @@ rc_t wait_for_and_release_background_file_merger( background_file_merger_t * sel
 /* called from the background-thread */
 static rc_t process_background_file_merger( background_file_merger_t * self ) {
     char tmp_filename[ 4096 ];
-    
+
     rc_t rc = generate_bg_merge_filename( self -> temp_dir, tmp_filename, sizeof tmp_filename,
                                           self -> product_id );
     if ( 0 == rc ) {
-        rc = Add_File_to_Cleanup_Task ( self -> cleanup_task, tmp_filename );
+        rc = clt_add_file( self -> cleanup_task, tmp_filename );
     }
     if ( 0 == rc ) {
         uint32_t num_src = 0;
@@ -664,7 +665,7 @@ static rc_t process_background_file_merger( background_file_merger_t * self ) {
                                         self -> dir,
                                         tmp_filename,   /* the output file */
                                         NULL,           /* opt. index_filename */
-                                        batch_files,    /* the input files */    
+                                        batch_files,    /* the input files */
                                         self -> buf_size,
                                         num_src,
                                         self -> gap );
@@ -674,7 +675,7 @@ static rc_t process_background_file_merger( background_file_merger_t * self ) {
                 }
             }
             if ( 0 == rc ) {
-                rc = delete_files( self -> dir, batch_files, self -> details );
+                rc = ft_delete_files( self -> dir, batch_files, self -> details );
             }
             VNamelistRelease( batch_files );
         }
@@ -703,14 +704,15 @@ static rc_t process_final_background_file_merger( background_file_merger_t * sel
                 if ( 0 == rc ) {
                     num_src++;
                 }
+				StringWhack( filename );
             }
         }
-        
+
         if ( 0 == rc ) {
-            rc = Add_File_to_Cleanup_Task ( self -> cleanup_task, self -> lookup_filename );
+            rc = clt_add_file( self -> cleanup_task, self -> lookup_filename );
         }
         if ( 0 == rc ) {
-            rc = Add_File_to_Cleanup_Task ( self -> cleanup_task, self -> index_filename );
+            rc = clt_add_file( self -> cleanup_task, self -> index_filename );
         }
         if ( 0 == rc ) {
             merge_sorter_t sorter;
@@ -718,7 +720,7 @@ static rc_t process_final_background_file_merger( background_file_merger_t * sel
                                     self -> dir,
                                     self -> lookup_filename,   /* the output file */
                                     self -> index_filename,    /* opt. index_filename */
-                                    batch_files,               /* the input files */    
+                                    batch_files,               /* the input files */
                                     self -> buf_size,
                                     num_src,
                                     self -> gap );
@@ -731,7 +733,7 @@ static rc_t process_final_background_file_merger( background_file_merger_t * sel
             }
         }
         if ( 0 == rc ) {
-            rc = delete_files( self -> dir, batch_files, self -> details );
+            rc = ft_delete_files( self -> dir, batch_files, self -> details );
         }
         VNamelistRelease( batch_files );
     }
@@ -804,10 +806,10 @@ rc_t make_background_file_merger( background_file_merger_t ** merger,
             rc = locked_value_init( &( b -> sealed ), 0 );
         }
         if ( 0 == rc ) {
-            rc = helper_make_thread( &( b -> thread ),
-                                     background_file_merger_thread_func,
-                                     b,
-                                     THREAD_DFLT_STACK_SIZE );
+            rc = hlp_make_thread( &( b -> thread ),
+                                  background_file_merger_thread_func,
+                                  b,
+                                  THREAD_DFLT_STACK_SIZE );
             if ( 0 != rc ) {
                 ErrMsg( "merge_sorter.c helper_make_thread( file-mergerr ) -> %R", rc );
             }
