@@ -34,6 +34,14 @@
 #include "temp_dir.h"
 #endif
 
+#ifndef _h_helper_
+#include "helper.h"
+#endif
+
+#ifndef _h_arg_helper_
+#include "arg_helper.h"
+#endif
+
 #ifndef _h_file_tools_
 #include "file_tools.h"
 #endif
@@ -54,13 +62,40 @@
 #include <klib/printf.h>
 #endif
 
+#ifndef _h_klib_namelist_
+#include <klib/namelist.h>
+#endif
+
+#include <klib/report.h> /* ReportResetObject */
+#include <vdb/report.h> /* ReportSetVDBManager */
+
+#ifndef _h_kproc_task_
+#include <kproc/task.h>
+#endif
+
 #ifndef _h_dflt_defline_
 #include "dflt_defline.h"
 #endif
 
-static rc_t print_tool_ctx( const tool_ctx_t * tool_ctx ) {
+bool tctx_populate_cmn_iter_params( const tool_ctx_t * tool_ctx,
+                                        cmn_iter_params_t * params ) {
+    bool res = false;
+    if ( NULL != tool_ctx && NULL != params ) {
+        res = cmn_iter_populate_params( params,
+                tool_ctx -> dir,
+                tool_ctx -> vdb_mgr,
+                tool_ctx -> accession_short,
+                tool_ctx -> accession_path,
+                tool_ctx -> cursor_cache,
+                0,
+                0 ); /* in cmn_iter.c */
+    }
+    return res;
+}
+
+static rc_t tctx_print( const tool_ctx_t * tool_ctx ) {
     rc_t rc = KOutHandlerSetStdErr();
-    
+
     if ( 0 == rc ) {
         rc = KOutMsg( "cursor-cache : %,lu bytes\n", tool_ctx -> cursor_cache );
     }
@@ -83,27 +118,27 @@ static rc_t print_tool_ctx( const tool_ctx_t * tool_ctx ) {
         rc = KOutMsg( "total ram    : %,lu bytes\n", tool_ctx -> total_ram );
     }
     if ( 0 == rc ) {
-        rc = KOutMsg( "output-format: %s\n", fmt_2_string( tool_ctx -> fmt ) );
+        rc = KOutMsg( "output-format: %s\n", hlp_fmt_2_string( tool_ctx -> fmt ) );
     }
     if ( 0 == rc ) {
-        rc = KOutMsg( "check-mode   : %s\n", check_mode_2_string( tool_ctx -> check_mode ) );
+        rc = KOutMsg( "check-mode   : %s\n", hlp_check_mode_2_string( tool_ctx -> check_mode ) );
     }
     if ( 0 == rc ) {
         rc = KOutMsg( "output-file  : '%s'\n",
                     NULL != tool_ctx -> output_filename ? tool_ctx -> output_filename : "-" );
     }
     if ( 0 == rc ) {
-        rc = KOutMsg( "output-dir   : '%s'\n", 
+        rc = KOutMsg( "output-dir   : '%s'\n",
                     NULL != tool_ctx -> output_dirname ? tool_ctx -> output_dirname : "." );
     }
     if ( 0 == rc ) {
         rc = KOutMsg( "output       : '%s'\n", tool_ctx -> dflt_output ); /* cannot be NULL */
     }
     if ( 0 == rc ) {
-        rc = KOutMsg( "append-mode  : '%s'\n", yes_or_no( tool_ctx -> append ) );
+        rc = KOutMsg( "append-mode  : '%s'\n", hlp_yes_or_no( tool_ctx -> append ) );
     }
     if ( 0 == rc ) {
-        rc = KOutMsg( "stdout-mode  : '%s'\n", yes_or_no( tool_ctx -> use_stdout ) );
+        rc = KOutMsg( "stdout-mode  : '%s'\n", hlp_yes_or_no( tool_ctx -> use_stdout ) );
     }
     if ( 0 == rc ) {
         rc = KOutMsg( "seq-defline  : '%s'\n", tool_ctx -> seq_defline );
@@ -112,10 +147,10 @@ static rc_t print_tool_ctx( const tool_ctx_t * tool_ctx ) {
         rc = KOutMsg( "qual-defline  : '%s'\n", tool_ctx -> qual_defline );
     }
     if ( 0 == rc ) {
-        rc = KOutMsg( "only-unaligned : '%s'\n", yes_or_no( tool_ctx -> only_unaligned ) );
+        rc = KOutMsg( "only-unaligned : '%s'\n", hlp_yes_or_no( tool_ctx -> only_unaligned ) );
     }
     if ( 0 == rc ) {
-        rc = KOutMsg( "only-aligned   : '%s'\n", yes_or_no( tool_ctx -> only_aligned ) );
+        rc = KOutMsg( "only-aligned   : '%s'\n", hlp_yes_or_no( tool_ctx -> only_aligned ) );
     }
     if ( 0 == rc ) {
         rc = KOutMsg( "accession     : '%s'\n", tool_ctx -> accession_short );
@@ -138,69 +173,96 @@ static rc_t print_tool_ctx( const tool_ctx_t * tool_ctx ) {
     if ( 0 == rc ) {
         rc = KOutMsg( "disk-limit-tmp (OS)  : %,lu bytes\n", tool_ctx -> disk_limit_tmp_os );
     }
-
-    if ( 0 == rc ) {    
-        rc = KOutMsg( "out/tmp on same fs   : '%s'\n", yes_or_no( tool_ctx -> out_and_tmp_on_same_fs ) );
+    if ( 0 == rc ) {
+        rc = KOutMsg( "out/tmp on same fs   : '%s'\n", hlp_yes_or_no( tool_ctx -> out_and_tmp_on_same_fs ) );
     }
+    if ( ft_fasta_ref_tbl == tool_ctx -> fmt ) {
+        if ( 0 == rc ) {        
+            rc = KOutMsg( "only internal ref    : '%s'\n", hlp_yes_or_no( tool_ctx -> only_internal_refs ) );
+        }
+        if ( 0 == rc ) {
+            rc = KOutMsg( "only external ref    : '%s'\n", hlp_yes_or_no( tool_ctx -> only_external_refs ) );
+        }
+        if ( 0 == rc ) {
+            rc = KOutMsg( "use name             : '%s'\n", hlp_yes_or_no( tool_ctx -> use_name ) );
+        }
+        if ( 0 == rc ) {
+            uint32_t idx, count = 0;
+            rc = VNameListCount( tool_ctx -> ref_name_filter, &count );
+            for ( idx = 0; 0 == rc && idx < count; ++idx ) {
+                const char * name = NULL;
+                rc = VNameListGet( tool_ctx -> ref_name_filter, idx, &name );
+                if ( 0 == rc && NULL != name ) {
+                    rc = KOutMsg( "\tref[%u] : '%s'\n", idx, name );
+                }
+            }
+        }
+    }
+
     if ( 0 == rc ) {
         rc = KOutMsg( "\n" );
     }
     if ( 0 == rc ) {
-        rc = inspection_report( &( tool_ctx -> insp_input ), &( tool_ctx -> insp_output ) ); /* inspector.c */
+        rc = insp_report( &( tool_ctx -> insp_input ), &( tool_ctx -> insp_output ) );
     }
     KOutHandlerSetStdOut();
     return rc;
 }
 
-static bool output_exists_whole( const tool_ctx_t * tool_ctx ) {
-    return file_exists( tool_ctx -> dir, "%s", tool_ctx -> output_filename ); /* file_tools.c */
+static bool tctx_output_exists_whole( const tool_ctx_t * tool_ctx ) {
+    bool res = ft_file_exists( tool_ctx -> dir, "%s", tool_ctx -> output_filename );
+    if ( !res ) {
+        res = ft_dir_exists( tool_ctx -> dir, "%s", tool_ctx -> output_filename );
+    }
+    return res;
 }
 
-static bool output_exists_idx( const tool_ctx_t * tool_ctx, uint32_t idx ) {
+static bool tctx_output_exists_idx( const tool_ctx_t * tool_ctx, uint32_t idx ) {
     bool res = false;
     SBuffer_t s_filename;
-   
+
     rc_t rc = split_filename_insert_idx( &s_filename, 4096,
                             tool_ctx -> output_filename, idx ); /* sbuffer.c */
     if ( 0 == rc ) {
-        res = file_exists( tool_ctx -> dir, "%S", &( s_filename . S ) ); /* file_tools.c */
+        res = ft_file_exists( tool_ctx -> dir, "%S", &( s_filename . S ) );
+        if ( !res ) {
+            res = ft_dir_exists( tool_ctx -> dir, "%S", &( s_filename . S ) );
+        }
         release_SBuffer( &s_filename ); /* helper.c */
     }
     return res;
 }
 
-static bool output_exists_split( const tool_ctx_t * tool_ctx ) {
-    bool res = output_exists_whole( tool_ctx );
+static bool tctx_output_exists_split( const tool_ctx_t * tool_ctx ) {
+    bool res = tctx_output_exists_whole( tool_ctx );
     if ( !res ) {
-        res = output_exists_idx( tool_ctx, 1 );
+        res = tctx_output_exists_idx( tool_ctx, 1 );
     }
     if ( !res ) {
-        res = output_exists_idx( tool_ctx, 2 );
+        res = tctx_output_exists_idx( tool_ctx, 2 );
     }
     return res;
 }
 
-static rc_t check_output_exits( const tool_ctx_t * tool_ctx ) {
+static rc_t tctx_check_output_exits( tool_ctx_t * tool_ctx ) {
     rc_t rc = 0;
-    /* check if the output-file(s) do already exist, in case we are not overwriting */    
+    /* check if the output-file(s) do already exist, in case we are not overwriting */
     if ( !( tool_ctx -> force ) && !( tool_ctx -> append ) ) {
         bool exists = false;
         switch( tool_ctx -> fmt ) {
-            case ft_unknown             : break;
-            case ft_fastq_whole_spot    : exists = output_exists_whole( tool_ctx ); break;
-            case ft_fastq_split_spot    : exists = output_exists_whole( tool_ctx ); break;
-            case ft_fastq_split_file    : exists = output_exists_split( tool_ctx ); break;
-            case ft_fastq_split_3       : exists = output_exists_split( tool_ctx ); break;
-            case ft_fasta_whole_spot    : exists = output_exists_whole( tool_ctx ); break;
-            case ft_fasta_split_spot    : exists = output_exists_whole( tool_ctx ); break;
-            case ft_fasta_us_split_spot : exists = output_exists_whole( tool_ctx ); break;
-            case ft_fasta_split_file    : exists = output_exists_split( tool_ctx ); break;
-            case ft_fasta_split_3       : exists = output_exists_split( tool_ctx ); break;
+            case ft_fastq_whole_spot    : exists = tctx_output_exists_whole( tool_ctx ); break;
+            case ft_fastq_split_spot    : exists = tctx_output_exists_whole( tool_ctx ); break;
+            case ft_fastq_split_file    : exists = tctx_output_exists_split( tool_ctx ); break;
+            case ft_fastq_split_3       : exists = tctx_output_exists_split( tool_ctx ); break;
+            case ft_fasta_whole_spot    : exists = tctx_output_exists_whole( tool_ctx ); break;
+            case ft_fasta_split_spot    : exists = tctx_output_exists_whole( tool_ctx ); break;
+            case ft_fasta_us_split_spot : exists = tctx_output_exists_whole( tool_ctx ); break;
+            case ft_fasta_split_file    : exists = tctx_output_exists_split( tool_ctx ); break;
+            case ft_fasta_split_3       : exists = tctx_output_exists_split( tool_ctx ); break;
+            default : break;
         }
         if ( exists ) {
-            rc = RC( rcExe, rcFile, rcPacking, rcName, rcExists );
-            ErrMsg( "fasterq-dump.c fastdump_csra() checking ouput-file '%s' -> %R",
-                     tool_ctx -> output_filename, rc );
+            tool_ctx -> force = true;
         }
     }
     return rc;
@@ -210,10 +272,10 @@ static rc_t check_output_exits( const tool_ctx_t * tool_ctx ) {
 #define MIN_NUM_THREADS 2
 #define MIN_MEM_LIMIT ( 1024L * 1024 * 5 )
 #define MAX_BUF_SIZE ( 1024L * 1024 * 1024 )
-static rc_t tool_ctx_encforce_constrains( tool_ctx_t * tool_ctx ) {
+static rc_t tctx_encforce_constrains( tool_ctx_t * tool_ctx ) {
     rc_t rc = 0;
     bool ignore_stdout = false;
-    uint32_t env_thread_count = get_env_u32( "DLFT_THREAD_COUNT", 0 );
+    uint32_t env_thread_count = ahlp_get_env_u32( "DLFT_THREAD_COUNT", 0 );
     if ( env_thread_count > 0  ) {
         tool_ctx -> num_threads = env_thread_count;
     } else {
@@ -230,8 +292,6 @@ static rc_t tool_ctx_encforce_constrains( tool_ctx_t * tool_ctx ) {
     }
     if ( tool_ctx -> use_stdout ) {
         switch( tool_ctx -> fmt ) {
-            case ft_unknown             : break;
-
             case ft_fastq_whole_spot    : break;
             case ft_fastq_split_spot    : break;
             case ft_fastq_split_file    : tool_ctx -> use_stdout = false; ignore_stdout = true; break;
@@ -242,6 +302,8 @@ static rc_t tool_ctx_encforce_constrains( tool_ctx_t * tool_ctx ) {
             case ft_fasta_us_split_spot : break;
             case ft_fasta_split_file    : tool_ctx -> use_stdout = false; ignore_stdout = true; break;
             case ft_fasta_split_3       : tool_ctx -> use_stdout = false; ignore_stdout = true; break;
+            
+            default : break;
         }
     }
     if ( tool_ctx -> use_stdout ) {
@@ -255,17 +317,20 @@ static rc_t tool_ctx_encforce_constrains( tool_ctx_t * tool_ctx ) {
     if ( ignore_stdout ) {
         rc = RC( rcExe, rcFile, rcPacking, rcName, rcExists );
         ErrMsg( "directing output to stdout requested." );
-        ErrMsg( "but requested mode ( %s ) would produce multiple files", fmt_2_string( tool_ctx -> fmt ) );
+        ErrMsg( "but requested mode ( %s ) would produce multiple files", hlp_fmt_2_string( tool_ctx -> fmt ) );
     }
     return rc;
 }
 
-rc_t release_tool_ctx( const tool_ctx_t * tool_ctx, rc_t rc_in ) {
+rc_t tctx_release( const tool_ctx_t * tool_ctx, rc_t rc_in ) {
     rc_t rc = rc_in;
+	
+	KTaskRelease( ( KTask* )tool_ctx -> cleanup_task );
+		
     if ( NULL != tool_ctx -> dir ) {
         rc_t rc2 = KDirectoryRelease( tool_ctx -> dir );
         if ( 0 != rc2 ) {
-            ErrMsg( "KDirectoryRelease() -> %R", rc2 );
+            ErrMsg( "release_tool_ctx . KDirectoryRelease() -> %R", rc2 );
             rc = ( 0 == rc ) ? rc2 : rc;
         }
     }
@@ -273,19 +338,29 @@ rc_t release_tool_ctx( const tool_ctx_t * tool_ctx, rc_t rc_in ) {
     if ( NULL != tool_ctx -> vdb_mgr ) {
         rc_t rc2 = VDBManagerRelease( tool_ctx -> vdb_mgr );
         if ( 0 != rc2 ) {
-            ErrMsg( "VDBManagerRelease() -> %R", rc2 );
+            ErrMsg( "release_tool_ctx . VDBManagerRelease() -> %R", rc2 );
             rc = ( 0 == rc ) ? rc2 : rc;
         }
     }
     if ( NULL != tool_ctx -> accession_short ) {
         free( ( char * )tool_ctx -> accession_short );
     }
+    if ( NULL != tool_ctx -> ref_name_filter ) {
+        rc_t rc2 = VNamelistRelease( tool_ctx -> ref_name_filter );
+        if ( 0 != rc2 ) {
+            ErrMsg( "release_tool_ctx . VNamelistRelease() -> %R", rc2 );
+            rc = ( 0 == rc ) ? rc2 : rc;
+        }
+    }
     return rc;
 }
 
-static rc_t tool_ctx_extract_short_accession( tool_ctx_t * tool_ctx ) {
+static rc_t tctx_extract_short_accession( tool_ctx_t * tool_ctx ) {
     rc_t rc = 0;
-    tool_ctx -> accession_short = inspector_extract_acc_from_path( tool_ctx -> accession_path ); /* inspector.c */
+    tool_ctx -> accession_short = insp_extract_acc_from_path( tool_ctx -> accession_path ); /* inspector.c */
+    ReportResetObject(  tool_ctx -> accession_path );
+    ReportBuildDate( __DATE__ );
+    ReportSetVDBManager( tool_ctx -> vdb_mgr );
 
     if ( NULL == tool_ctx -> accession_short ) {
         rc = RC( rcApp, rcArgv, rcAccessing, rcParam, rcInvalid );
@@ -294,7 +369,7 @@ static rc_t tool_ctx_extract_short_accession( tool_ctx_t * tool_ctx ) {
     return rc;
 }
 
-static rc_t tool_ctx_create_lookup_and_index_path( tool_ctx_t * tool_ctx ) {
+static rc_t tctx_create_lookup_and_index_path( tool_ctx_t * tool_ctx ) {
     rc_t rc = generate_lookup_filename( tool_ctx -> temp_dir,
                                         &tool_ctx -> lookup_filename[ 0 ],
                                         sizeof tool_ctx -> lookup_filename ); /* temp_dir.c */
@@ -302,7 +377,7 @@ static rc_t tool_ctx_create_lookup_and_index_path( tool_ctx_t * tool_ctx ) {
         ErrMsg( "fasterq-dump.c handle_lookup_path( lookup_filename ) -> %R", rc );
     } else {
         size_t num_writ;
-        /* generate the full path of the lookup-index-table */                
+        /* generate the full path of the lookup-index-table */
         rc = string_printf( &tool_ctx -> index_filename[ 0 ], sizeof tool_ctx -> index_filename,
                             &num_writ,
                             "%s.idx",
@@ -315,7 +390,7 @@ static rc_t tool_ctx_create_lookup_and_index_path( tool_ctx_t * tool_ctx ) {
 }
 
 /* we have NO output-dir and NO output-file */
-static rc_t tool_ctx_make_output_filename_from_accession( tool_ctx_t * tool_ctx, bool fasta ) {
+static rc_t tctx_make_output_filename_from_accession( tool_ctx_t * tool_ctx, bool fasta ) {
     /* we DO NOT have a output-directory : build output-filename from the accession */
     /* generate the full path of the output-file, if not given */
     rc_t rc = KDirectoryResolvePath( tool_ctx -> dir,
@@ -324,7 +399,7 @@ static rc_t tool_ctx_make_output_filename_from_accession( tool_ctx_t * tool_ctx,
                                 sizeof tool_ctx -> dflt_output,
                                 "%s%s",
                                 tool_ctx -> accession_short,
-                                out_ext( fasta ) /* helper.c */ );
+                                hlp_out_ext( fasta ) /* helper.c */ );
     if ( 0 != rc ) {
         ErrMsg( "tool_ctx_make_output_filename_from_accession.KDirectoryResolvePath() -> %R", rc );
     } else {
@@ -334,8 +409,8 @@ static rc_t tool_ctx_make_output_filename_from_accession( tool_ctx_t * tool_ctx,
 }
 
 /* we have an output-dir and NO output-file */
-static rc_t tool_ctx_make_output_filename_from_dir_and_accession( tool_ctx_t * tool_ctx, bool fasta ) {
-    bool es = ends_in_slash( tool_ctx -> output_dirname ); /* helper.c */
+static rc_t tctx_make_output_filename_from_dir_and_accession( tool_ctx_t * tool_ctx, bool fasta ) {
+    bool es = hlp_ends_in_slash( tool_ctx -> output_dirname ); /* helper.c */
     rc_t rc = KDirectoryResolvePath( tool_ctx -> dir,
                                 true /* absolute */,
                                 &( tool_ctx -> dflt_output[ 0 ] ),
@@ -343,7 +418,7 @@ static rc_t tool_ctx_make_output_filename_from_dir_and_accession( tool_ctx_t * t
                                 es ? "%s%s%s" : "%s/%s%s",
                                 tool_ctx -> output_dirname,
                                 tool_ctx -> accession_short,
-                                out_ext( fasta ) /* helper.c */ );
+                                hlp_out_ext( fasta ) /* helper.c */ );
     if ( 0 != rc ) {
         ErrMsg( "tool_ctx_make_output_filename_from_dir_and_accession.KDirectoryResolvePath() -> %R", rc );
     } else {
@@ -352,20 +427,20 @@ static rc_t tool_ctx_make_output_filename_from_dir_and_accession( tool_ctx_t * t
     return rc;
 }
 
-static rc_t tool_ctx_optionally_create_paths_in_output_filename( tool_ctx_t * tool_ctx ) {
+static rc_t tctx_optionally_create_paths_in_output_filename( tool_ctx_t * tool_ctx ) {
     rc_t rc = 0;
     String path;
-    if ( extract_path( tool_ctx -> output_filename, &path ) ) {
+    if ( hlp_extract_path( tool_ctx -> output_filename, &path ) ) {
         /* the output-filename contains a path... */
-        if ( !dir_exists( tool_ctx -> dir, "%S", &path ) ) {
+        if ( !ft_dir_exists( tool_ctx -> dir, "%S", &path ) ) {
             /* this path does not ( yet ) exist, create it... */
-            rc = create_this_dir( tool_ctx -> dir, &path, true );
+            rc = ft_create_this_dir( tool_ctx -> dir, &path, true );
         }
     }
     return rc;
 }
 
-static rc_t tool_ctx_resolve_output_filename( tool_ctx_t * tool_ctx ) {
+static rc_t tctx_resolve_output_filename( tool_ctx_t * tool_ctx ) {
     rc_t rc = KDirectoryResolvePath( tool_ctx -> dir,
                                 true /* absolute */,
                                 &( tool_ctx -> dflt_output[ 0 ] ),
@@ -373,28 +448,28 @@ static rc_t tool_ctx_resolve_output_filename( tool_ctx_t * tool_ctx ) {
                                 "%s",
                                 tool_ctx -> output_filename );
     if ( 0 != rc ) {
-        ErrMsg( "tool_ctx_resolve_output_filename.KDirectoryResolvePath() -> %R", rc );        
+        ErrMsg( "tool_ctx_resolve_output_filename.KDirectoryResolvePath() -> %R", rc );
     } else {
-        tool_ctx -> output_filename = tool_ctx -> dflt_output;        
-    }
-    return rc;
-}
-    
-static rc_t tool_ctx_adjust_output_filename( tool_ctx_t * tool_ctx ) {
-    rc_t rc = 0;
-    /* we do have a output-filename : use it */
-    if ( dir_exists( tool_ctx -> dir, "%s", tool_ctx -> output_filename ) ) { /* helper.c */
-        /* the given output-filename is an existing directory ! */
-        rc = RC( rcVDB, rcNoTarg, rcConstructing, rcParam, rcInvalid );
-        ErrMsg( "string_printf( output-filename ) -> %R", rc );
-    } else {
-        rc = tool_ctx_optionally_create_paths_in_output_filename( tool_ctx ); /* above */
+        tool_ctx -> output_filename = tool_ctx -> dflt_output;
     }
     return rc;
 }
 
-static rc_t tool_ctx_adjust_output_filename_by_dir( tool_ctx_t * tool_ctx ) {
-    bool es = ends_in_slash( tool_ctx -> output_dirname ); /* helper.c */
+static rc_t tctx_adjust_output_filename( tool_ctx_t * tool_ctx ) {
+    rc_t rc = 0;
+    /* we do have a output-filename : use it */
+    if ( ft_dir_exists( tool_ctx -> dir, "%s", tool_ctx -> output_filename ) ) { /* helper.c */
+        /* the given output-filename is an existing directory ! */
+        rc = RC( rcVDB, rcNoTarg, rcConstructing, rcParam, rcInvalid );
+        ErrMsg( "output >%s< already exists", tool_ctx -> output_filename );
+    } else {
+        rc = tctx_optionally_create_paths_in_output_filename( tool_ctx ); /* above */
+    }
+    return rc;
+}
+
+static rc_t tctx_adjust_output_filename_by_dir( tool_ctx_t * tool_ctx ) {
+    bool es = hlp_ends_in_slash( tool_ctx -> output_dirname ); /* helper.c */
     rc_t rc = KDirectoryResolvePath( tool_ctx -> dir,
                                 true /* absolute */,
                                 &( tool_ctx -> dflt_output[ 0 ] ),
@@ -406,28 +481,28 @@ static rc_t tool_ctx_adjust_output_filename_by_dir( tool_ctx_t * tool_ctx ) {
         ErrMsg( "tool_ctx_adjust_output_filename_by_dir.KDirectoryResolvePath() -> %R", rc );
     } else {
         tool_ctx -> output_filename = tool_ctx -> dflt_output;
-        rc = tool_ctx_optionally_create_paths_in_output_filename( tool_ctx );
+        rc = tctx_optionally_create_paths_in_output_filename( tool_ctx );
     }
     return rc;
 }
 
-static void tool_ctx_get_disk_limits( tool_ctx_t * tool_ctx ) {
+static void tctx_get_disk_limits( tool_ctx_t * tool_ctx ) {
     /* we do not stop processing if there is an error while asking for this value */
-    available_space_disk_space( tool_ctx -> dir,
+    ft_available_space_disk_space( tool_ctx -> dir,
                                 tool_ctx -> output_filename,
                                 &( tool_ctx -> disk_limit_out_os ),
                                 true /* is_file */ ); /* file_tools.c */
     if ( NULL != tool_ctx -> temp_dir ) {
-        available_space_disk_space( tool_ctx -> dir,
+        ft_available_space_disk_space( tool_ctx -> dir,
                                     get_temp_dir( tool_ctx -> temp_dir ), /* tmp_dir.c */
                                     &( tool_ctx -> disk_limit_tmp_os ),
                                     false /* is_file */ ); /* file_tools.c */
     }
 }
 
-static size_t tool_ctx_temp_file_sizes( tool_ctx_t * tool_ctx ) {
+static size_t tctx_temp_file_sizes( tool_ctx_t * tool_ctx ) {
     size_t res = 0;
-    
+
     /* if the accession is not local, each thread creates it's own local cache of it */
     if ( tool_ctx -> insp_output . is_remote ) {
         res = ( tool_ctx -> insp_output . acc_size * tool_ctx -> num_threads );
@@ -452,18 +527,18 @@ static size_t tool_ctx_get_temp_file_limit( tool_ctx_t * tool_ctx ) {
     return res;
 }
 
-static size_t tool_ctx_get_out_file_limit( tool_ctx_t * tool_ctx ) {
+static size_t tctx_get_out_file_limit( tool_ctx_t * tool_ctx ) {
     size_t res = tool_ctx -> disk_limit_out_cmdl;   /* the commandline has priority! */
     if ( 0 == res ) { res = tool_ctx -> disk_limit_out_os; } /* as a last resort, if above has not value */
     return res;
 }
 
-static rc_t tool_ctx_check_available_disk_size( tool_ctx_t * tool_ctx ) {
+static rc_t tctx_check_available_disk_size( tool_ctx_t * tool_ctx ) {
     rc_t rc = 0;
 
     if ( tool_ctx -> use_stdout ) {
         /* if the tool is set to output on stdout: we only need to check the temp-file-size */
-        size_t needed = tool_ctx_temp_file_sizes( tool_ctx ); /* above */
+        size_t needed = tctx_temp_file_sizes( tool_ctx ); /* above */
         /* the limit is either given at the command-line or queried from the OS */
         size_t limit = tool_ctx_get_temp_file_limit( tool_ctx );
         if ( limit > 0 && needed > limit ) {
@@ -471,13 +546,13 @@ static rc_t tool_ctx_check_available_disk_size( tool_ctx_t * tool_ctx ) {
             rc = RC( rcVDB, rcNoTarg, rcConstructing, rcParam, rcTooBig );
         }
     } else {
-        /* we have to check for output- as well as for temp-file-size */        
+        /* we have to check for output- as well as for temp-file-size */
         if ( tool_ctx -> out_and_tmp_on_same_fs ) {
             /* output as well as temp-files are on the same file-system, temp-size includes the output */
-            size_t needed_tmp = tool_ctx_temp_file_sizes( tool_ctx ); /* above */
+            size_t needed_tmp = tctx_temp_file_sizes( tool_ctx ); /* above */
             size_t needed_out = tool_ctx -> estimated_output_size;
             size_t needed = needed_tmp > needed_out ? needed_tmp : needed_out;
-            size_t limit = tool_ctx_get_out_file_limit( tool_ctx );
+            size_t limit = tctx_get_out_file_limit( tool_ctx );
             if ( 0 == limit ) { limit = tool_ctx_get_temp_file_limit( tool_ctx ); }
             if ( limit > 0 && needed > limit ) {
                 /* we are over the limit! */
@@ -485,10 +560,10 @@ static rc_t tool_ctx_check_available_disk_size( tool_ctx_t * tool_ctx ) {
             }
         } else {
             /* output and temp-files are on different file-systems */
-            size_t needed_tmp = tool_ctx_temp_file_sizes( tool_ctx ); /* above */
+            size_t needed_tmp = tctx_temp_file_sizes( tool_ctx ); /* above */
             size_t needed_out = tool_ctx -> estimated_output_size;
             size_t needed = needed_tmp > needed_out ? needed_tmp : needed_out;
-            size_t limit = tool_ctx_get_out_file_limit( tool_ctx ); /* above */
+            size_t limit = tctx_get_out_file_limit( tool_ctx ); /* above */
             if ( limit > 0 && needed > limit ) {
                 /* we are over the limit! */
                 rc = RC( rcVDB, rcNoTarg, rcConstructing, rcParam, rcTooBig );
@@ -501,11 +576,10 @@ static rc_t tool_ctx_check_available_disk_size( tool_ctx_t * tool_ctx ) {
             }
         }
     }
-    
+
     if ( 0 == rc ) {
         if ( tool_ctx -> show_details ) {
             rc_t rc2 = KOutHandlerSetStdErr();
-            
             if ( 0 == rc2 ) {
                 rc2 = KOutMsg( "disk-limit(s) not exeeded!\n" );
                 KOutHandlerSetStdOut();
@@ -513,11 +587,10 @@ static rc_t tool_ctx_check_available_disk_size( tool_ctx_t * tool_ctx ) {
         }
     } else {
         rc_t rc2 = KOutHandlerSetStdErr();
-        
         if ( 0 == rc2 ) {
             rc2 = KOutMsg( "disk-limit exeeded!\n" );
             if ( 0 == rc2 && !tool_ctx -> show_details ) {
-                rc2 = KOutMsg( "to see limits: re-run with '-x' option.\n" );                
+                rc2 = KOutMsg( "to see limits: re-run with '-x' option.\n" );
             }
             KOutHandlerSetStdOut();
         }
@@ -525,12 +598,52 @@ static rc_t tool_ctx_check_available_disk_size( tool_ctx_t * tool_ctx ) {
     return rc;
 }
 
+static bool tctx_does_format_produce_reads_in_a_single_file( format_t fmt ) {
+    switch ( fmt ) {
+        case ft_unknown             : return false; break;
+        case ft_fastq_whole_spot    : return false; break;
+        case ft_fastq_split_spot    : return true; break;
+        case ft_fastq_split_file    : return false; break;
+        case ft_fastq_split_3       : return false; break;
+        case ft_fasta_whole_spot    : return false; break;
+        case ft_fasta_split_spot    : return true; break;
+        case ft_fasta_split_file    : return false; break;
+        case ft_fasta_split_3       : return false; break;
+        case ft_fasta_us_split_spot : return true; break;
+        case ft_fasta_ref_tbl       : return true; break;
+        case ft_fasta_concat        : return true; break;        
+        case ft_ref_report          : return true; break;
+    }
+    return false;
+}
+
+
+static rc_t tctx_check_available_columns( tool_ctx_t * tool_ctx ) {
+    rc_t rc = 0;
+    
+    if ( !hlp_is_format_fasta( tool_ctx -> fmt ) ) {
+        /* FASTQ is requested... */
+        if ( !tool_ctx -> insp_output . seq . has_quality_column ) {
+            rc = RC( rcApp, rcArgv, rcAccessing, rcParam, rcInvalid );
+            ErrMsg( "the input data is missing the QUALITY-column" );
+        }
+    }
+    if ( 0 == rc ) {
+        /* we need READ in any case... */
+        if ( !tool_ctx -> insp_output . seq . has_read_column ) {
+            rc = RC( rcApp, rcArgv, rcAccessing, rcParam, rcInvalid );
+            ErrMsg( "the input data is missing the READ-column" );
+        }
+    }
+    return rc;
+}
+
 /* taken form libs/kapp/main-priv.h */
 rc_t KAppGetTotalRam ( uint64_t * totalRam );
 
-rc_t populate_tool_ctx( tool_ctx_t * tool_ctx ) {
+rc_t tctx_populate_and_call_inspector( tool_ctx_t * tool_ctx ) {
 
-    bool fasta = is_format_fasta( tool_ctx -> fmt ); /* helper.c */
+    bool fasta = hlp_is_format_fasta( tool_ctx -> fmt ); /* helper.c */
 
     /* create the KDirectory-instance for all modules to use */
     rc_t rc = KDirectoryNativeDir( &( tool_ctx -> dir ) );
@@ -556,30 +669,30 @@ rc_t populate_tool_ctx( tool_ctx_t * tool_ctx ) {
 
     /* enforce some constrains: thread-count, mem-limit, buffer-size, stdout, only-aligned/unaligned */
     if ( 0 == rc ) {
-        rc = tool_ctx_encforce_constrains( tool_ctx ); /* above */
+        rc = tctx_encforce_constrains( tool_ctx );
     }
 
     /* extract the accesion-string, for output- and temp-files to use */
     if ( 0 == rc ) {
-        rc = tool_ctx_extract_short_accession( tool_ctx ); /* above */
+        rc = tctx_extract_short_accession( tool_ctx );
     }
 
     /* create the temp. directory ( only if we need it! ) */
     if ( 0 == rc && tool_ctx -> fmt != ft_fasta_us_split_spot ) {
         rc = make_temp_dir( &tool_ctx -> temp_dir,
                         tool_ctx -> requested_temp_path,
-                        tool_ctx -> dir ); /* temp_dir.c */
+                        tool_ctx -> dir );
     }
 
     /* create the lookup- and index-filenames ( only if we need it! ) */
     if ( 0 == rc && tool_ctx -> fmt != ft_fasta_us_split_spot ) {
-        rc = tool_ctx_create_lookup_and_index_path( tool_ctx ); /* above */
+        rc = tctx_create_lookup_and_index_path( tool_ctx );
     }
 
     /* if an output-directory is explicity given from the commandline: create if not exists */
     if ( 0 == rc && NULL != tool_ctx -> output_dirname && cmt_only != tool_ctx -> check_mode ) {
-        if ( !dir_exists( tool_ctx -> dir, "%s", tool_ctx -> output_dirname ) ) /* file_tools.c */ {
-            rc = create_this_dir_2( tool_ctx -> dir, tool_ctx -> output_dirname, true ); /* file_tools.c */
+        if ( !ft_dir_exists( tool_ctx -> dir, "%s", tool_ctx -> output_dirname ) ) {
+            rc = ft_create_this_dir_2( tool_ctx -> dir, tool_ctx -> output_dirname, true );
         }
     }
 
@@ -588,32 +701,32 @@ rc_t populate_tool_ctx( tool_ctx_t * tool_ctx ) {
         if ( NULL == tool_ctx -> output_filename ) {
             /* no output-filename has been given on the commandline */
             if ( NULL == tool_ctx -> output_dirname ) {
-                rc = tool_ctx_make_output_filename_from_accession( tool_ctx, fasta ); /* above */
+                rc = tctx_make_output_filename_from_accession( tool_ctx, fasta );
             } else {
-                rc = tool_ctx_make_output_filename_from_dir_and_accession( tool_ctx, fasta ); /* above */
+                rc = tctx_make_output_filename_from_dir_and_accession( tool_ctx, fasta );
             }
         } else {
             /* there is an output-filename on the commandline */
             if ( NULL == tool_ctx -> output_dirname ) {
-                rc = tool_ctx_resolve_output_filename( tool_ctx );
+                rc = tctx_resolve_output_filename( tool_ctx );
                 if ( 0 == rc ) {
-                    rc = tool_ctx_adjust_output_filename( tool_ctx ); /* above */
+                    rc = tctx_adjust_output_filename( tool_ctx );
                 }
             } else {
-                rc = tool_ctx_adjust_output_filename_by_dir( tool_ctx ); /* above */
+                rc = tctx_adjust_output_filename_by_dir( tool_ctx );
             }
         }
         if ( 0 == rc && cmt_only != tool_ctx -> check_mode ) {
-            rc = check_output_exits( tool_ctx ); /* above */
+            rc = tctx_check_output_exits( tool_ctx );
         }
     }
 
     /* create the cleanup-taks ( for modules to add file/directories to it ) and add the tem-dir to it */
     tool_ctx -> cleanup_task = NULL;
     if ( 0 == rc && tool_ctx -> fmt != ft_fasta_us_split_spot ) {
-        rc = Make_FastDump_Cleanup_Task ( &( tool_ctx -> cleanup_task ), tool_ctx -> show_details ); /* cleanup_task.c */
+        rc = clt_create( &( tool_ctx -> cleanup_task ), tool_ctx -> show_details ); /* cleanup_task.c */
         if ( 0 == rc ) {
-            rc = Add_Directory_to_Cleanup_Task ( tool_ctx -> cleanup_task, 
+            rc = clt_add_directory( tool_ctx -> cleanup_task, 
                     get_temp_dir( tool_ctx -> temp_dir ) ); /* cleanup_task.c */
         }
     }
@@ -624,30 +737,53 @@ rc_t populate_tool_ctx( tool_ctx_t * tool_ctx ) {
         tool_ctx -> insp_input . vdb_mgr = tool_ctx -> vdb_mgr;
         tool_ctx -> insp_input . accession_short = tool_ctx -> accession_short;
         tool_ctx -> insp_input . accession_path = tool_ctx -> accession_path;
-
+        tool_ctx -> insp_input . requested_seq_tbl_name = tool_ctx -> requested_seq_tbl_name;
+        
         rc = inspect( &( tool_ctx -> insp_input ), &( tool_ctx -> insp_output ) ); /* inspector.c */
     }
 
+    /* check for presence of columns for certain output-types FASTA vs. FASTQ */
+    if ( 0 == rc ) {
+        rc = tctx_check_available_columns( tool_ctx );
+    }
+    
     /* create seq/qual deflines ( if they are not given at the commandline ) */
     if ( 0 == rc ) {
         bool has_name = tool_ctx -> insp_output . seq . has_name_column;
         bool use_name = true;
-        bool use_read_id = false;
+
+        /* use_read_id should be true if the output is a single file */
+        bool use_read_id = tctx_does_format_produce_reads_in_a_single_file( tool_ctx -> fmt );
 
         if ( NULL == tool_ctx -> seq_defline ) {
-            tool_ctx -> seq_defline  = dflt_seq_defline( has_name, use_name, use_read_id, fasta );
+            tool_ctx -> seq_defline  = dflt_seq_defline( has_name, use_name, use_read_id, fasta ); /* dflt_defline.c */
         }
         if ( NULL == tool_ctx -> qual_defline && !fasta ) {
-            tool_ctx -> qual_defline = dflt_qual_defline( has_name, use_name, use_read_id );
+            tool_ctx -> qual_defline = dflt_qual_defline( has_name, use_name, use_read_id ); /* dflt_defline.c */
         }
     }
 
+    /* special check if we have a combination of split-3 and include-technical */
+    if ( 0 == rc ) {
+        bool split_3_requested = ( ft_fasta_split_3 == tool_ctx -> fmt || ft_fastq_split_3 == tool_ctx -> fmt );
+        bool include_technical = !( tool_ctx -> join_options . skip_tech );
+        if ( split_3_requested && include_technical ) {
+            /* warn the user, and switch to split-file-mode */
+            StdErrMsg( "split-3-mode cannot be combined with include-technical -> switching to split-file-mode\n" );
+            switch ( tool_ctx -> fmt ) {
+                case ft_fasta_split_3 : tool_ctx -> fmt = ft_fasta_split_file; break;
+                case ft_fastq_split_3 : tool_ctx -> fmt = ft_fastq_split_file; break;
+                default : ;
+            }
+        }
+    }
+        
     /* evaluate the free-disk-space according the os */
-    if ( 0 == rc ) { tool_ctx_get_disk_limits( tool_ctx ); /* above */  }
-    
+    if ( 0 == rc ) { tctx_get_disk_limits( tool_ctx ); }
+
     /* create an estimation of the output-size */
-    if ( 0 == rc && is_perform_check( tool_ctx -> check_mode ) /* helper.c */ ) {
-        inspector_estimate_input_t iei; /* inspector.h */
+    if ( 0 == rc && hlp_is_perform_check( tool_ctx -> check_mode ) ) {
+        insp_estimate_input_t iei;
 
         iei . insp = &( tool_ctx -> insp_output );
         iei . seq_defline  = tool_ctx -> seq_defline;
@@ -658,24 +794,26 @@ rc_t populate_tool_ctx( tool_ctx_t * tool_ctx ) {
         iei . avg_tech_reads = tool_ctx -> insp_output . seq . avg_tech_reads;
         iei . skip_tech = tool_ctx -> join_options . skip_tech;
         iei . fmt = tool_ctx -> fmt;
-        
-        tool_ctx -> estimated_output_size = inspector_estimate_output_size( &iei );
+
+        tool_ctx -> estimated_output_size = insp_estimate_output_size( &iei );
     }
 
+    /* determine if output and temp. path are on the same file-systme ( work is in helper-function ) */
     if ( 0 == rc ) {
-        tool_ctx -> out_and_tmp_on_same_fs = paths_on_same_filesystem(
-            tool_ctx -> output_filename, get_temp_dir( tool_ctx -> temp_dir ) ); /* helper.c */
+        tool_ctx -> out_and_tmp_on_same_fs = hlp_paths_on_same_filesystem(
+            tool_ctx -> output_filename, get_temp_dir( tool_ctx -> temp_dir ) );
     }
-    
+
     /* print all the values gathered here, if requested */
     if ( 0 == rc && tool_ctx -> show_details ) {
-        rc = print_tool_ctx( tool_ctx ); /* above */
+        rc = tctx_print( tool_ctx );
     }
 
+    /* check if we have enough space, based on the check-mode */
     if ( 0 == rc ) {
         switch( tool_ctx -> check_mode ) { /* check-mode defined in helper.h */
             case cmt_on     :
-            case cmt_only   : rc = tool_ctx_check_available_disk_size( tool_ctx ); break; /* above */
+            case cmt_only   : rc = tctx_check_available_disk_size( tool_ctx ); break;
             default         : break;
         }
     }
