@@ -34,6 +34,8 @@
 #include <klib/log.h>
 #endif
 
+#include <klib/report.h> /* ReportResetObject */
+
 #ifndef _h_vfs_manager_
 #include <vfs/manager.h>
 #endif
@@ -53,8 +55,8 @@ static const char * ref_usage[] = { "Filter by position on genome.",
 static const char * outf_usage[] = { "Output will be written to this file",
                               "instead of std-out", NULL };
 
-static const char * table_usage[] = { "Which alignment table(s) to use (p|s|e):", 
-                               "p - primary, s - secondary, e - evidence-interval", 
+static const char * table_usage[] = { "Which alignment table(s) to use (p|s|e):",
+                               "p - primary, s - secondary, e - evidence-interval",
                                "(default = p)", NULL };
 
 static const char * gzip_usage[] = { "Compress output using gzip", NULL };
@@ -103,7 +105,7 @@ OptDef CommonOptions[] = {
     { OPTION_BZIP,    ALIAS_BZIP,    NULL, bzip_usage,    1,        false,       false },
     { OPTION_INF,     ALIAS_INF,     NULL, inf_usage,     0,        true,        false },
     { OPTION_SCHEMA,  ALIAS_SCHEMA,  NULL, schema_usage,  1,        true,        false },
-    { OPTION_NO_MT,   NULL,          NULL, no_mt_usage,   1,        false,       false },  
+    { OPTION_NO_MT,   NULL,          NULL, no_mt_usage,   1,        false,       false },
     { OPTION_TIMING,  NULL,          NULL, timing_usage,  1,        true,        false }
 };
 
@@ -399,7 +401,7 @@ static rc_t split_argument_into_path_and_readgroup( const char *argument, char *
         rc = split_argument( argument, path, attribute, '=' );
     } else {
         VFSManager * mgr;
-        rc_t rc = VFSManagerMake ( & mgr );
+        rc = VFSManagerMake ( & mgr );
 
         *path = NULL;
         *attribute = NULL;
@@ -521,6 +523,8 @@ static rc_t prepare_db_table( prepare_ctx *ctx,
     ctx->db = NULL;
     ctx->seq_tab = NULL;
 
+    ReportResetObject ( path );
+
     rc = VDBManagerOpenDBRead ( vdb_mgr, &ctx->db, vdb_schema, "%s", path );
     if ( rc != 0 ) {
         rc = VDBManagerOpenTableRead ( vdb_mgr, &ctx->seq_tab, NULL, "%s", path );
