@@ -140,19 +140,6 @@ static bool on_set_location_error( Dlg &dlg, ESetRootState s ) {
 /* ==== pick public location ================================================================= */
 static std::string public_location_start_dir( vdbconf_model * model ) {
     std::string res = model -> get_public_location();
-
-    if ( !model -> does_path_exist( res ) ) {
-        res = model -> get_user_default_dir();
-    }
-    if ( !model -> does_path_exist( res ) ) {
-        res = model -> get_home_dir() + "/ncbi";
-    }
-    if ( !model -> does_path_exist( res ) ) {
-        res = model -> get_home_dir();
-    }
-    if ( !model -> does_path_exist( res ) ) {
-        res = model -> get_current_dir();
-    }
     return res;
 }
 
@@ -165,6 +152,14 @@ static bool pick_public_location( Dlg &dlg, vdbconf_model * model ) {
     } else {
 		res = input_ctrl::input( dlg, "location of public cache", path, 256 );
     }
+    if ( res ) { 
+        res = path.length() > 0;
+    }
+    if ( res ) {
+        res = model -> set_public_location_direct( path );
+    }
+    /*
+     * No more unneccesary complexity!
     if ( res && path.length() > 0 ) {
         std::ostringstream q;
         q << "do you want to change the location to '" << path << "' ?";
@@ -190,6 +185,7 @@ static bool pick_public_location( Dlg &dlg, vdbconf_model * model ) {
             }
         }
     }
+    */
     return res;
 }
 
@@ -624,10 +620,10 @@ class vdbconf_ctrl : public Dlg_Runner {
 #endif
 
                 case CACHE_USE_CACHE_ID  : view.status_txt( "use local cache" ); break;
-                case CACHE_REPO_CHOOSE_ID : view.status_txt( "choose loacation of local repository" ); break;
-                case CACHE_REPO_CLEAR_ID  : view.status_txt( "clear loacation of local repository" ); break;
-                case CACHE_PROC_CHOOSE_ID : view.status_txt( "choose loacation of process local storage" ); break;
-                case CACHE_PROC_CLEAR_ID  : view.status_txt( "clear loacation of process local storage" ); break;
+                case CACHE_REPO_CHOOSE_ID : view.status_txt( "choose location of local repository" ); break;
+                case CACHE_REPO_CLEAR_ID  : view.status_txt( "clear location of local repository" ); break;
+                case CACHE_PROC_CHOOSE_ID : view.status_txt( "choose location of process local storage" ); break;
+                case CACHE_PROC_CLEAR_ID  : view.status_txt( "clear location of process local storage" ); break;
 
                 case AWS_CHARGES_ID : view.status_txt( "do accept charges for AWS usage" ); break;
                 case AWS_REPORT_ID  : view.status_txt( "report cloud instance identity to ..." ); break;
