@@ -91,7 +91,7 @@ static const char *USAGE_MD5[] = { "Check components md5s if present, "
 #define ALIAS_blob_crc  "b"
 #define OPTION_blob_crc "blob-crc"
 static const char *USAGE_BLOB_CRC[] =
-{ "Check blobs CRC32 (default: yes)", NULL };
+{ "Check blobs CRC32 (default: no)", NULL };
 
 #define ALIAS_BLOB_CRC  "B"
 #define OPTION_BLOB_CRC "BLOB-CRC"
@@ -149,6 +149,10 @@ static const char *USAGE_IND_ONLY[] =
 static const char *USAGE_CHECK_REDACT[] =
 { "check if redaction of bases has been correctly performed (default: false)", NULL };
 
+#define OPTION_REQUIRE_BLOB_CRC "require-blob-checksums"
+static const char *USAGE_REQUIRE_BLOB_CRC[] = 
+{ "Require blob checksums (default: no)", NULL };
+
 static OptDef options [] =
 {                                                    /* needs_value, required */
 /*  { OPTION_MD5     , ALIAS_MD5     , NULL, USAGE_MD5     , 1, true , false }*/
@@ -177,6 +181,8 @@ static OptDef options [] =
   , { OPTION_ref_int , ALIAS_ref_int , NULL, USAGE_REF_INT , 1, false, false }
 
   , { OPTION_CHECK_REDACT, NULL      , NULL, USAGE_CHECK_REDACT, 1, false , false }
+
+  , { OPTION_REQUIRE_BLOB_CRC, NULL  , NULL, USAGE_REQUIRE_BLOB_CRC, 1, false , false }
 };
 
 /*
@@ -227,6 +233,8 @@ rc_t CC Usage ( const Args * args )
     HelpOptionLine(NULL          , OPTION_NGC           , "path", USAGE_NGC);
 
     HelpOptionLine(NULL          , OPTION_CHECK_REDACT, NULL, USAGE_CHECK_REDACT);
+
+    HelpOptionLine(NULL          , OPTION_REQUIRE_BLOB_CRC, NULL, USAGE_REQUIRE_BLOB_CRC);
 /*
 #define NUM_LISTABLE_OPTIONS \
     ( sizeof options / sizeof options [ 0 ] - NUM_SILENT_TRAILING_OPTIONS )
@@ -256,8 +264,9 @@ rc_t parse_args ( vdb_validate_params *pb, Args *args )
 
     pb -> md5_chk = true;
     pb->consist_check = false;
-    ref_int_check = pb -> blob_crc
+    ref_int_check
         = pb -> md5_chk_explicit = md5_required = true;
+    pb -> blob_crc = false;
     pb -> sdc_sec_rows_in_percent = false;
     pb -> sdc_sec_rows.number = 100000;
     pb -> sdc_seq_rows_in_percent = false;
@@ -297,6 +306,12 @@ rc_t parse_args ( vdb_validate_params *pb, Args *args )
           return rc;
       pb -> check_redact = ( cnt != 0 );
   }
+  {
+      rc = ArgsOptionCount ( args, OPTION_REQUIRE_BLOB_CRC, & cnt );
+      if ( rc != 0 )
+          return rc;
+      pb -> blob_crc_required = ( cnt != 0 );
+  }
   
   {
     rc = ArgsOptionCount(args, OPTION_REF_INT, &cnt);
@@ -354,8 +369,8 @@ rc_t parse_args ( vdb_validate_params *pb, Args *args )
             return rc;
         }
         assert(dummy && dummy[0]);
-        if (dummy[0] == 'n') {
-            pb -> blob_crc = false;
+        if (dummy[0] == 'y') {
+            pb -> blob_crc = true;
         }
     }
   }
@@ -431,58 +446,58 @@ rc_t parse_args ( vdb_validate_params *pb, Args *args )
       }
   }
   {
-        rc = ArgsOptionCount ( args, OPTION_SDC_SEQ_ROWS, &cnt );
+    rc = ArgsOptionCount ( args, OPTION_SDC_SEQ_ROWS, &cnt );
+    if (rc)
+    {
+        LOGERR (klogInt, rc, "ArgsOptionCount() failed for " OPTION_SDC_SEQ_ROWS);
+        return rc;
+    }
+
+    if (cnt > 0)
+    {
+        uint64_t value;
+        size_t value_size;
+        rc = ArgsOptionValue ( args, OPTION_SDC_SEQ_ROWS, 0, (const void **) &dummy );
         if (rc)
         {
-            LOGERR (klogInt, rc, "ArgsOptionCount() failed for " OPTION_SDC_SEQ_ROWS);
+            LOGERR (klogInt, rc, "ArgsOptionValue() failed for " OPTION_SDC_SEQ_ROWS);
             return rc;
         }
 
-        if (cnt > 0)
+        pb->sdc_enabled = true;
+
+        value_size = string_size ( dummy );
+        if ( value_size >= 1 && dummy[value_size - 1] == '%' )
         {
-            uint64_t value;
-            size_t value_size;
-            rc = ArgsOptionValue ( args, OPTION_SDC_SEQ_ROWS, 0, (const void **) &dummy );
+            value = string_to_U64 ( dummy, value_size - 1, &rc );
             if (rc)
             {
-                LOGERR (klogInt, rc, "ArgsOptionValue() failed for " OPTION_SDC_SEQ_ROWS);
+                LOGERR (klogInt, rc, "string_to_U64() failed for " OPTION_SDC_SEQ_ROWS);
+                return rc;
+            }
+            else if (value == 0 || value > 100)
+            {
+                rc = RC(rcExe, rcArgv, rcParsing, rcParam, rcInvalid);
+                LOGERR (klogInt, rc, OPTION_SDC_SEQ_ROWS " has illegal percentage value (has to be 1-100%)" );
                 return rc;
             }
 
-            pb->sdc_enabled = true;
-
-            value_size = string_size ( dummy );
-            if ( value_size >= 1 && dummy[value_size - 1] == '%' )
+            pb->sdc_seq_rows_in_percent = true;
+            pb->sdc_seq_rows.percent = (double)value / 100;
+        }
+        else
+        {
+            value = string_to_U64 ( dummy, value_size, &rc );
+            if (rc)
             {
-                value = string_to_U64 ( dummy, value_size - 1, &rc );
-                if (rc)
-                {
-                    LOGERR (klogInt, rc, "string_to_U64() failed for " OPTION_SDC_SEQ_ROWS);
-                    return rc;
-                }
-                else if (value == 0 || value > 100)
-                {
-                    rc = RC(rcExe, rcArgv, rcParsing, rcParam, rcInvalid);
-                    LOGERR (klogInt, rc, OPTION_SDC_SEQ_ROWS " has illegal percentage value (has to be 1-100%)" );
-                    return rc;
-                }
-
-                pb->sdc_seq_rows_in_percent = true;
-                pb->sdc_seq_rows.percent = (double)value / 100;
-            }
-            else
-            {
-                value = string_to_U64 ( dummy, value_size, &rc );
-                if (rc)
-                {
-                    LOGERR (klogInt, rc, "string_to_U64() failed for " OPTION_SDC_SEQ_ROWS);
-                    return rc;
-                }
-                pb->sdc_seq_rows_in_percent = false;
-                pb->sdc_seq_rows.number = value;
+                LOGERR (klogInt, rc, "string_to_U64() failed for " OPTION_SDC_SEQ_ROWS);
+                return rc;
             }
+            pb->sdc_seq_rows_in_percent = false;
+            pb->sdc_seq_rows.number = value;
         }
     }
+  }
     {
         rc = ArgsOptionCount ( args, OPTION_SDC_PLEN_THOLD, &cnt );
         if (rc)
