@@ -52,7 +52,7 @@
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <string.h>
+#include <strings.h>
 #include <assert.h>
 
 /* make it last include */
