@@ -370,22 +370,6 @@ public:
         out << p_content;
     }
 
-    std::string GetDbMetadataAttr ( const std::string& p_node, const std::string& p_attr )
-    {
-        const KMetadata *meta;
-        THROW_ON_RC ( VDatabaseOpenMetadataRead ( m_db, &meta ) );
-
-        const KMDataNode *node;
-        THROW_ON_RC ( KMetadataOpenNodeRead ( meta, &node, p_node.c_str() ) );
-
-        size_t num_read;
-        char attr [ 256 ];
-        THROW_ON_RC ( KMDataNodeReadAttr ( node, p_attr.c_str(), attr, sizeof attr, & num_read ) );
-        THROW_ON_RC ( KMDataNodeRelease ( node ) );
-        THROW_ON_RC ( KMetadataRelease ( meta ) );
-        return string ( attr, num_read );
-    }
-
     std::string GetMetadata ( const KMetadata* p_meta, const std::string& p_node )
     {
         const KMDataNode *node;
@@ -400,6 +384,22 @@ public:
         return string ( buf, num_read );
 
     }
+    std::string GetMetadataAttr ( const KMetadata* p_meta, const std::string& p_node, const std::string& p_attr )
+    {
+        const KMDataNode *node;
+        THROW_ON_RC ( KMetadataOpenNodeRead ( p_meta, &node, p_node.c_str() ) );
+
+        size_t num_read;
+        char buf[ 256 ];
+
+        THROW_ON_RC ( KMDataNodeReadAttr ( node, p_attr.c_str(),
+        buf, sizeof buf, &num_read ) );
+
+        THROW_ON_RC ( KMDataNodeRelease ( node ) );
+        return string ( buf, num_read );
+
+    }
+
     std::string GetDbMetadata ( VDatabase* p_db, const std::string& p_node )
     {
         const KMetadata *meta;
@@ -409,6 +409,23 @@ public:
         return ret;
     }
 
+    std::string GetDbMetadataAttr ( const VDatabase* p_db, const std::string& p_node, const std::string& p_attr )
+    {
+        const KMetadata *meta;
+        THROW_ON_RC ( VDatabaseOpenMetadataRead ( p_db, &meta ) );
+        string ret = GetMetadataAttr ( meta, p_node, p_attr );
+        THROW_ON_RC ( KMetadataRelease ( meta ) );
+        return ret;
+    }
+    std::string GetTblMetadataAttr ( const VTable* p_tbl, const std::string& p_node, const std::string& p_attr )
+    {
+        const KMetadata *meta;
+        THROW_ON_RC ( VTableOpenMetadataRead ( p_tbl, &meta ) );
+        string ret = GetMetadataAttr ( meta, p_node, p_attr );
+        THROW_ON_RC ( KMetadataRelease ( meta ) );
+        return ret;
+    }
+
     TestSource      m_source;
     VDatabase *     m_db;
     const VCursor * m_cursor;
@@ -599,8 +616,8 @@ FIXTURE_TEST_CASE ( SoftwareName, GeneralLoaderFixture )
 
     // validate metadata
     OpenDatabase ();
-    REQUIRE_EQ ( SoftwareName,  GetDbMetadataAttr ( "SOFTWARE/formatter", "name" ) );
-    REQUIRE_EQ ( Version,       GetDbMetadataAttr ( "SOFTWARE/formatter", "vers" ) );
+    REQUIRE_EQ ( SoftwareName,  GetDbMetadataAttr ( m_db, "SOFTWARE/formatter", "name" ) );
+    REQUIRE_EQ ( Version,       GetDbMetadataAttr ( m_db, "SOFTWARE/formatter", "vers" ) );
 
     // extract the program name from path the same way it's done in ncbi-vdb/libs/kapp/loader-meta.c:KLoaderMeta_Write
     {
@@ -613,14 +630,14 @@ FIXTURE_TEST_CASE ( SoftwareName, GeneralLoaderFixture )
             tool_name = argv0.c_str();
         }
 
-        REQUIRE_EQ ( string ( tool_name ), GetDbMetadataAttr ( "SOFTWARE/loader", "name" ) );
+        REQUIRE_EQ ( string ( tool_name ), GetDbMetadataAttr ( m_db, "SOFTWARE/loader", "name" ) );
     }
 
-    REQUIRE_EQ ( string ( __DATE__), GetDbMetadataAttr ( "SOFTWARE/loader", "date" ) );
+    REQUIRE_EQ ( string ( __DATE__), GetDbMetadataAttr ( m_db, "SOFTWARE/loader", "date" ) );
     {
         char buf[265];
         string_printf ( buf, sizeof buf, NULL, "%V", KAppVersion() ); // same format as in ncbi-vdb/libs/kapp/loader-meta.c:MakeVersion()
-        REQUIRE_EQ ( string ( buf ), GetDbMetadataAttr ( "SOFTWARE/loader", "vers" ) );
+        REQUIRE_EQ ( string ( buf ), GetDbMetadataAttr ( m_db, "SOFTWARE/loader", "vers" ) );
     }
 }
 
@@ -759,6 +776,22 @@ FIXTURE_TEST_CASE ( DBMetadataNode, GeneralLoaderFixture )
     REQUIRE_EQ ( value, GetDbMetadata( m_db, key ) );
 }
 
+FIXTURE_TEST_CASE ( DBMetadataNodeAttr, GeneralLoaderFixture )
+{
+    SetUpStream ( GetName() );
+    const string key = "dbmetadatanode";
+    const string attr = "attr";
+    const string value = "1a2b3c4d";
+    m_source . OpenStreamEvent();
+    m_source . DBMetadataNodeEvent ( 0, key, value );
+    m_source . DBMetadataNodeAttrEvent ( 0, key, attr, value );
+    m_source . CloseStreamEvent();
+    REQUIRE ( Run ( m_source . MakeSource (), 0 ) );
+
+    OpenDatabase ();
+    REQUIRE_EQ ( value, GetDbMetadataAttr( m_db, key, attr ) );
+}
+
 FIXTURE_TEST_CASE ( SubDBMetadataNode, GeneralLoaderFixture )
 {
     string schemaFile = ScratchDir + GetName() + ".vschema";
@@ -831,6 +864,42 @@ FIXTURE_TEST_CASE ( TblMetadataNode, GeneralLoaderFixture )
     }
 }
 
+FIXTURE_TEST_CASE ( TblMetadataNodeAttr, GeneralLoaderFixture )
+{
+    string schemaFile = ScratchDir + GetName() + ".vschema";
+    CreateFile ( schemaFile,
+                 string (
+                    "table table1 #1.0.0 { column ascii column1; };"
+                    "database root_database #1 { table table1 #1 TABLE1; } ;"
+                 )
+    );
+
+    const char* tblName = "tbl";
+    const uint32_t tblId = 2;
+    const string key = "tblmetadatanode";
+    const string attr = "tblattr";
+    const string value = "tbl1a2b3c4d";
+
+    SetUpStream ( GetName(), schemaFile, "root_database");
+
+    m_source . DBAddTableEvent ( tblId, 0, "TABLE1", tblName, kcmCreate );
+
+    m_source . OpenStreamEvent();
+    m_source . TblMetadataNodeEvent ( tblId, key, value);
+    m_source . TblMetadataNodeAttrEvent ( tblId, key, attr, value);
+    m_source . CloseStreamEvent();
+    REQUIRE ( Run ( m_source . MakeSource (), 0 ) );
+
+    // validate database
+    OpenDatabase ();
+    {
+        const VTable *tbl;
+        REQUIRE_RC ( VDatabaseOpenTableRead ( m_db, & tbl, tblName ) );
+        REQUIRE_EQ ( value, GetTblMetadataAttr ( tbl, key, attr ) );
+        REQUIRE_RC ( VTableRelease ( tbl ) );
+    }
+}
+
 FIXTURE_TEST_CASE ( ColMetadataNode, GeneralLoaderFixture )
 {
     string schemaFile = ScratchDir + GetName() + ".vschema";
@@ -891,6 +960,69 @@ FIXTURE_TEST_CASE ( ColMetadataNode, GeneralLoaderFixture )
     }
 }
 
+FIXTURE_TEST_CASE ( ColMetadataNodeAttr, GeneralLoaderFixture )
+{
+    string schemaFile = ScratchDir + GetName() + ".vschema";
+    CreateFile ( schemaFile,
+                 string (
+                    "table table1 #1.0.0 { column ascii column1; };"
+                    "database root_database #1 { table table1 #1 TABLE1; } ;"
+                 )
+    );
+
+    const char* tblName = "tbl";
+    const uint32_t tblId = 2;
+    const char* colName = "column1";
+    const uint32_t colId = 5;
+    const string NodeName   = "colmetadatanode";
+    const string NodeValue  = "1a2b3c4d";
+    const string AttrName   = "colattr";
+    const string AttrValue   = "colattr";
+
+    SetUpStream ( GetName(), schemaFile, "root_database");
+
+    m_source . DBAddTableEvent ( tblId, 0, "TABLE1", tblName, kcmCreate | kcmMD5 );
+    m_source . NewColumnEvent ( colId, tblId, colName, 8 );
+
+    m_source . OpenStreamEvent();
+    m_source . ColMetadataNodeEvent ( colId, NodeName, NodeValue );
+    m_source . ColMetadataNodeAttrEvent ( colId, NodeName, AttrName, AttrValue );
+    // need at least 2 rows with different value in order for the column to become physical, otherwise column's metadata will not be stored
+    m_source . CellDataEvent( colId, string("blah1") );
+    m_source . NextRowEvent ( tblId );
+    m_source . CellDataEvent( colId, string("brh2") );
+    m_source . NextRowEvent ( tblId );
+    m_source . CloseStreamEvent();
+    REQUIRE ( Run ( m_source . MakeSource (), 0 ) );
+
+    // validate metadata
+    OpenDatabase ();
+    {
+        const VTable *tbl;
+        REQUIRE_RC ( VDatabaseOpenTableRead ( m_db, & tbl, tblName ) );
+
+        {
+            const KTable* ktbl;
+            REQUIRE_RC ( VTableOpenKTableRead ( tbl, & ktbl ) );
+            {
+                const KColumn* col;
+                REQUIRE_RC ( KTableOpenColumnRead ( ktbl, & col, colName ) );
+                {
+                    const KMetadata *meta;
+                    REQUIRE_RC ( KColumnOpenMetadataRead ( col, &meta ) );
+
+                    REQUIRE_EQ ( AttrValue, GetMetadataAttr ( meta, NodeName, AttrName ) );
+
+                    REQUIRE_RC ( KMetadataRelease ( meta ) );
+                }
+                REQUIRE_RC ( KColumnRelease ( col ) );
+            }
+            REQUIRE_RC ( KTableRelease ( ktbl ) );
+        }
+        REQUIRE_RC ( VTableRelease ( tbl ) );
+    }
+}
+
 FIXTURE_TEST_CASE ( NoData, GeneralLoaderFixture )
 {
     SetUpStream ( GetName() );
