@@ -1,5 +1,4 @@
-#ifndef __FASTQ_WRITER_HPP__
-#define __FASTQ_WRITER_HPP__
+#pragma once
 
 /**
  * @file fastq_writer.hpp
@@ -43,10 +42,16 @@
 #include "fastq_read.hpp"
 #include "fastq_error.hpp"
 #include "sra-tools/writer.hpp"
+
+#include <fingerprint.hpp>
+
 #include <spdlog/spdlog.h>
 #include "spdlog/sinks/base_sink.h"
 #include <spdlog/sinks/stdout_sinks.h>
 #include <insdc/sra.h>
+#include <json.hpp>
+#include <fstream>
+
 #define LOCALDEBUG
 
 /**
@@ -74,6 +79,9 @@ public:
     }
 };
 
+class fastq_writer;
+
+
 /**
  * @brief Redirects spdlog logging messages to general-loader
  *
@@ -83,24 +91,25 @@ template<typename Mutex>
 class general_writer_sink : public spdlog::sinks::base_sink<Mutex>
 {
 public:
+/*
     general_writer_sink(shared_ptr<Writer2>& writer_)
         : //spdlog::sinks::base_sink<Mutex>(logger_name)
         writer(writer_)
     {}
-protected:
-    void sink_it_(const spdlog::details::log_msg& msg) override {
-        spdlog::memory_buf_t formatted;
-        spdlog::sinks::base_sink<Mutex>::formatter_->format(msg, formatted);
-        if (msg.level >= SPDLOG_LEVEL_ERROR)
-            writer->errorMessage(fmt::to_string(formatted));
-        else
-            writer->logMessage(fmt::to_string(formatted));
-        //writer->progressMessage(fmt::to_string(formatted));
+*/
+    general_writer_sink(fastq_writer* writer_)
+        : //spdlog::sinks::base_sink<Mutex>(logger_name)
+        m_writer(writer_)
+    {
+        assert(m_writer);
     }
+protected:
+    void sink_it_(const spdlog::details::log_msg& msg) override;
     void flush_() override {}
-    shared_ptr<Writer2> writer;
+    fastq_writer* m_writer = nullptr;
 };
 
+
 using general_writer_sink_mt = general_writer_sink<std::mutex>;
 
 /**
@@ -112,8 +121,9 @@ using general_writer_sink_mt = general_writer_sink<std::mutex>;
  * @return std::shared_ptr<spdlog::logger>
  */
 template<typename Factory = spdlog::synchronous_factory>
-inline std::shared_ptr<spdlog::logger> general_writer_logger_mt(const std::string &logger_name, shared_ptr<Writer2>& writer)
+inline std::shared_ptr<spdlog::logger> general_writer_logger_mt(const std::string &logger_name, fastq_writer* writer)
 {
+    assert(writer);
     // use this one to create a logger that sinks to general-loader only
     //auto gw_logger = Factory::template create<general_writer_sink_mt>(logger_name, writer);
 
@@ -153,7 +163,7 @@ public:
      *
      * @param[in] reads
      */
-    virtual void write_spot(const vector<CFastqRead>& reads);
+    virtual void write_spot(const string& spot_name, const vector<CFastqRead>& reads) = 0;
 
     /**
      * @brief Set user-defined attributes
@@ -164,33 +174,115 @@ public:
     void set_attr(const string& name, const string& value) {
         m_attr[name] = value;
     }
+
+    /**
+     * @brief Get the attr object
+     * value is not changed if attr is not found
+     *
+     * @param name
+     * @param value
+     */
+    void get_attr(const string& name, string& value) {
+        auto it = m_attr.find(name);
+        if (it != m_attr.end()) {
+            value = it->second;
+        }
+    }
+
+    void add_message(int level, const string& msg) {
+        lock_guard<mutex> lock(m_msg_mutex);
+        if (level >= SPDLOG_LEVEL_ERROR)
+            m_err_messages.push_back(msg);
+        else
+            m_log_messages.push_back(msg);
+        m_has_messages = true;
+    }
+
+    virtual void write_messages()
+    {
+        if (!m_has_messages)
+            return;
+        lock_guard<mutex> lock(m_msg_mutex);
+        for (const auto& msg : m_err_messages) {
+            cerr << msg << endl;
+        }
+        for (const auto& msg : m_log_messages) {
+            cout << msg << endl;
+        }
+        m_err_messages.clear();
+        m_log_messages.clear();
+        m_has_messages = false;
+    }
+
+    void set_fingerprint( const string & source, const Fingerprint & fp )
+    {
+        m_source_fp.push_back( make_pair(source, fp ) );
+    }
+
 protected:
     using TAttributeName = string;
     using TAttributeValue = string;
     map<TAttributeName, TAttributeValue> m_attr;  ///< Attributes dictionary
+    mutex m_msg_mutex; ///< Mutex for messages
+    vector<string> m_err_messages; ///< Messages
+    vector<string> m_log_messages; ///< Messages
+    atomic<bool> m_has_messages{false}; ///< Messages
+    vector< pair<string, Fingerprint> > m_source_fp; ///< read fingerprints per input file
+};
+
+class fastq_writer_debug : public fastq_writer
+{
+public:
+    fastq_writer_debug() {};
+
+    virtual void open() override
+    {
+        string quality_expression = "(INSDC:quality:text:phred_33)QUALITY";
+        get_attr("quality_expression", quality_expression);
+        m_quality_as_string = quality_expression == "(INSDC:quality:phred)QUALITY";
+    };
+
+    virtual void write_spot(const string& spot_name, const vector<CFastqRead>& reads) override
+    {
+        if (reads.empty())
+            return;
+        const auto& first_read = reads.front();
+        //string spot_name = first_read.Spot();
+        //spot_name += first_read.Suffix();
+        cout << "Spot: " << spot_name << first_read.Suffix() << "\nreads " << reads.size() <<":\n";
+        vector<uint8_t> qual_scores;
+        for (const auto& read : reads) {
+            //auto sz = read.Sequence().size();
+            cout << "num:" << read.ReadNum() << "(" << (read.Type() == 0 ? "T" : "B") << ")" << "\n";
+            //cout << "spot_group:" << read.SpotGroup() << "\n";
+            cout << read.Sequence() << "\n";
+            cout << "+\n";
+            qual_scores.clear();
+            read.GetQualScores(qual_scores);
+            if (qual_scores.empty())
+                continue;
+            if (m_quality_as_string) {
+                auto it = qual_scores.begin();
+                cout << to_string(*it);
+                ++it;
+                for (;it != qual_scores.end(); ++it) {
+                    cout << " " << to_string(*it);
+                }
+            } else {
+                for (const auto& q : qual_scores) {
+                    cout << q;
+                }
+            }
+            cout << endl;
+        }
+    }
+    bool m_quality_as_string{false};
+    using fastq_writer::m_source_fp;
 };
 
 
-void fastq_writer::write_spot(const vector<CFastqRead>& reads)
-{
-    if (reads.empty())
-        return;
-    const auto& first_read = reads.front();
-    string spot_name = first_read.Spot();
-    spot_name += first_read.Suffix();
-    cout << "Spot: " << spot_name << "\nreads " << reads.size() <<":\n";
-    for (const auto& read : reads) {
-        //auto sz = read.Sequence().size();
-        cout << "num:" << read.ReadNum() << "(" << (read.Type() == 0 ? "T" : "B") << ")" << "\n";
-        cout << read.Sequence() << "\n";
-        cout << "+\n";
-        cout << read.Quality() << endl;
-    }
-}
-
-
 /**
- * @brief VDB Writer implementaion
+ * @brief VDB Writer implementation
  *
  * Constructor redirects logging to general_loader
  * Open() method sets up the VDB table using user defin attributes
@@ -205,12 +297,31 @@ public:
 
     void open() override;
     void close() override;
-    void write_spot(const vector<CFastqRead>& reads) override;
+    virtual void write_spot(const string& spot_name, const vector<CFastqRead>& reads) override;
 
     const Writer2 & get_writer() const { return * m_writer; }
-private:
+    virtual void write_messages() override
+    {
+        if (!m_has_messages)
+            return;
+        lock_guard<mutex> lock(m_msg_mutex);
+        m_has_messages = false;
+        for (const auto& msg : m_err_messages) {
+            m_writer->errorMessage(msg);
+        }
+        m_err_messages.clear();
+        for (const auto& msg : m_log_messages) {
+            m_writer->logMessage(msg);
+        }
+        m_log_messages.clear();
+
+    }
+
+    const Fingerprint & get_read_fingerprint() const { return m_read_fingerprint; }
+
+protected:
     shared_ptr<Writer2> m_writer;    ///< VDB Writer
-    std::shared_ptr<spdlog::logger> m_default_logger; ///< Saved default loger
+    std::shared_ptr<spdlog::logger> m_default_logger; ///< Saved default logger
     Writer2::Table SEQUENCE_TABLE;
     Writer2::Column c_NAME;
     Writer2::Column c_SPOT_GROUP;
@@ -225,6 +336,12 @@ private:
     Writer2::Column c_READ_NUMBER;
     uint8_t m_platform{0};
     bool m_is_writing{false};  ///< Flag to indicate if writing was initiated
+    string m_tmp_sequence; ///< temp string for sequences
+    string m_tmp_spot; ///< temp string for spots
+    vector<uint8_t> m_qual_scores; ///< temp vector for scores
+    vector<char> mReadTypes;
+
+    Fingerprint m_read_fingerprint;
 };
 
 //  -----------------------------------------------------------------------------
@@ -240,7 +357,7 @@ fastq_writer_vdb::fastq_writer_vdb(ostream& stream, shared_ptr<Writer2> writer =
     }
 
     m_default_logger = spdlog::default_logger();
-    auto logger = general_writer_logger_mt("general_writer", m_writer);
+    auto logger = general_writer_logger_mt("general_writer", this);
     spdlog::set_default_logger(logger);
 }
 
@@ -251,77 +368,78 @@ fastq_writer_vdb::~fastq_writer_vdb()
         close();
     }
 
+
 }
 
 //  -----------------------------------------------------------------------------
 void fastq_writer_vdb::open()
 {
-    static const string cSCHEMA = "sra/generic-fastq.vschema";
-    static const string cGENERIC_DB = "NCBI:SRA:GenericFastq:db";
-    static const string cILLUMINA_DB = "NCBI:SRA:Illumina:db";
-    static const string cNANOPORE_DB = "NCBI:SRA:GenericFastqNanopore:db";
+    //static const string cSCHEMA = "sra/generic-fastq.vschema";
+    //static const string cGENERIC_DB = "NCBI:SRA:GenericFastq:db";
 
     string name_column_expression = "RAW_NAME";
-    string name_column = "RAW_NAME";
-    {
-        auto name_column_it = m_attr.find("name_column");
-        if (name_column_it != m_attr.end()) {
-            name_column = name_column_it->second;
-            name_column_expression.clear();
-            if (name_column == "NAME") {
-                name_column_expression = "(ascii)";
-            }
-            name_column_expression += name_column;
-        }
+    get_attr("name_column", name_column_expression);
+    bool has_name_column{false};
+    if (name_column_expression == "NAME") {
+        has_name_column = true;
+        name_column_expression = "(ascii)NAME";
     }
 
     string quality_expression = "(INSDC:quality:text:phred_33)QUALITY";
-    {
-        auto qual_it = m_attr.find("quality_expression");
-        if (qual_it != m_attr.end()) {
-            quality_expression = qual_it->second;
-        }
-    }
+    get_attr("quality_expression", quality_expression);
 
-    {
-        auto platform_it = m_attr.find("platform");
-        if (platform_it != m_attr.end()) {
-            m_platform = stoi(platform_it->second);
-        }
-    }
+    string platform;
+    get_attr("platform", platform);
+    if (!platform.empty())
+        m_platform = stoi(platform);
 
+    string destination{"sra.out"};
+    get_attr("destination", destination);
+    m_writer->destination(destination);
+
+    string version{"1.0"};
+    get_attr("version", version);
+    m_writer->info("sharq", version);
+
+    static const string cGENERIC_SCHEMA = "sra/generic-fastq.vschema";
+    static const string c454_SCHEMA = "sra/454.vschema";
+    static const string cION_TORRENT_SCHEMA = "sra/ion-torrent.vschema";
+
+    static const string cGENERIC_DB = "NCBI:SRA:GenericFastq:db";
+    static const string cILLUMINA_DB = "NCBI:SRA:Illumina:db";
+    static const string cNANOPORE_DB = "NCBI:SRA:GenericFastqNanopore:db";
+    static const string c454_DB = "NCBI:SRA:_454_:db";
+    static const string cION_TORRENT_DB = "NCBI:SRA:IonTorrent:db";
     string db = cGENERIC_DB;
-    switch ( m_platform )
+    string schema = cGENERIC_SCHEMA;
+    switch (m_platform)
     {
     case SRA_PLATFORM_ILLUMINA:
         // use Illumina DB for illumina with standard NAME column
-        if (name_column == "NAME")
+        if (has_name_column)
             db = cILLUMINA_DB;
         break;
     case SRA_PLATFORM_OXFORD_NANOPORE:
         db = cNANOPORE_DB;
         break;
+    case SRA_PLATFORM_454:
+        schema = c454_SCHEMA;
+        db = c454_DB;
+        break;
+    case SRA_PLATFORM_ION_TORRENT:
+        schema = cION_TORRENT_SCHEMA;
+        db = cION_TORRENT_DB;
+        break;
     }
+    //m_writer->set_attr("schema", schema);
+    //m_writer->set_attr("db", db);
 
-    string destination{"sra.out"};
-    {
-        auto dest_it = m_attr.find("destination");
-        if (dest_it != m_attr.end()) {
-            destination = dest_it->second;
-        }
-    }
 
-    string version{"1.0"};
-    {
-        auto dest_it = m_attr.find("version");
-        if (dest_it != m_attr.end()) {
-            version = dest_it->second;
-        }
-    }
-
-    m_writer->destination(destination);
-    m_writer->schema(cSCHEMA, db);
-    m_writer->info("sharq", version);
+    //string schema = cSCHEMA;
+    //string db = cGENERIC_DB;
+    //get_attr("schema", schema);
+    //get_attr("db", db);
+    m_writer->schema(schema, db);
 
     vector<Writer2::ColumnDefinition> SequenceCols = {
         { "READ",               sizeof(char) }, // sequence literals
@@ -348,22 +466,26 @@ void fastq_writer_vdb::open()
     };
 
     m_writer->addTable("SEQUENCE", SequenceCols);
-    SEQUENCE_TABLE = move(m_writer->table("SEQUENCE"));
-    c_NAME = move(SEQUENCE_TABLE.column("NAME"));
-    c_SPOT_GROUP = move(SEQUENCE_TABLE.column("SPOT_GROUP"));
-    c_PLATFORM = move(SEQUENCE_TABLE.column("PLATFORM"));
-    c_READ = move(SEQUENCE_TABLE.column("READ"));
-    c_QUALITY = move(SEQUENCE_TABLE.column("QUALITY"));
-    c_READ_START = move(SEQUENCE_TABLE.column("READ_START"));
-    c_READ_LEN = move(SEQUENCE_TABLE.column("READ_LEN"));
-    c_READ_TYPE = move(SEQUENCE_TABLE.column("READ_TYPE"));
-    c_READ_FILTER = move(SEQUENCE_TABLE.column("READ_FILTER"));
+    SEQUENCE_TABLE = m_writer->table("SEQUENCE");
+    c_NAME = SEQUENCE_TABLE.column("NAME");
+    c_SPOT_GROUP = SEQUENCE_TABLE.column("SPOT_GROUP");
+    c_PLATFORM = SEQUENCE_TABLE.column("PLATFORM");
+    c_READ = SEQUENCE_TABLE.column("READ");
+    c_QUALITY = SEQUENCE_TABLE.column("QUALITY");
+    c_READ_START = SEQUENCE_TABLE.column("READ_START");
+    c_READ_LEN = SEQUENCE_TABLE.column("READ_LEN");
+    c_READ_TYPE = SEQUENCE_TABLE.column("READ_TYPE");
+    c_READ_FILTER = SEQUENCE_TABLE.column("READ_FILTER");
     if ( m_platform == SRA_PLATFORM_OXFORD_NANOPORE )
     {
-        c_CHANNEL = move(SEQUENCE_TABLE.column("CHANNEL"));
-        c_READ_NUMBER = move(SEQUENCE_TABLE.column("READ_NUMBER"));
+        c_CHANNEL = SEQUENCE_TABLE.column("CHANNEL");
+        c_READ_NUMBER = SEQUENCE_TABLE.column("READ_NUMBER");
     }
 
+    string read_types;
+    get_attr("readTypes", read_types);
+    mReadTypes = {read_types.begin(), read_types.end()};
+
     m_writer->beginWriting();
     m_is_writing = true;
 
@@ -373,11 +495,27 @@ void fastq_writer_vdb::open()
 void fastq_writer_vdb::close()
 {
     if (m_is_writing && m_writer) {
+
+        // save fingerprints in the metadata
+        // input fingerprint(s)
+        for( size_t i = 0;  i < m_source_fp.size(); ++i )
+        {
+            ostringstream key;
+            key << "LOAD/QC/file_" << (i+1);
+            m_writer->setMetadata( VDB::Writer::MetaNodeRoot::database, 0, key.str(), m_source_fp[i].second.JSON() );
+            m_writer->setMetadataAttr( VDB::Writer::MetaNodeRoot::database, 0, key.str(), "name", m_source_fp[i].first );
+        }
+
+        {   // output fingerprint
+            Writer2::TableID SequenceTabId = m_writer->table("SEQUENCE").id();
+            m_writer->setMetadata( VDB::Writer::MetaNodeRoot::table, SequenceTabId, "QC/fingerprint", m_read_fingerprint.JSON() );
+        }
+
+        write_messages();
         m_writer->endWriting();
         m_is_writing = false;
         m_writer->flush();
         spdlog::set_default_logger(m_default_logger);
-        //spdlog::set_default_logger(m_default_logger);
         //m_writer.reset();
     }
 }
@@ -396,32 +534,41 @@ enum
 };
 */
 //  -----------------------------------------------------------------------------
-void fastq_writer_vdb::write_spot(const vector<CFastqRead>& reads)
+void fastq_writer_vdb::write_spot(const string& spot_name, const vector<CFastqRead>& reads)
 {
     if (reads.empty())
         return;
     //auto table = m_writer->table("SEQUENCE");
+
     const auto& first_read = reads.front();
-    string spot_name = first_read.Spot();
-    spot_name += first_read.Suffix();
-    c_NAME.setValue(spot_name);
+/*
+    m_tmp_spot = first_read.Spot();
+*/
+    m_tmp_spot = spot_name;
+    m_tmp_spot += first_read.Suffix();
+    c_NAME.setValue(m_tmp_spot);
+
     c_SPOT_GROUP.setValue(first_read.SpotGroup());
     c_PLATFORM.setValue(m_platform);
-    string sequence;
-    vector<uint8_t>  qual_scores;
     auto read_num = reads.size();
-    int32_t read_start[read_num];
     size_t start  = 0;
+    int32_t read_start[read_num];
     int32_t read_len[read_num];
     char read_type[read_num];
     char read_filter[read_num];
     uint32_t channel[read_num];
     uint32_t read_no[read_num];
-
+    m_tmp_sequence.clear();
+    m_qual_scores.clear();
     read_num = 0;
+
+    //if (read_num >= mReadTypes.size())
+        //throw fastq_error(30, "readTypes number should match the number of reads {} != {}", mReadTypes.size(), read_num);
+
     for (const auto& read : reads) {
-        sequence += read.Sequence();
-        read.GetQualScores(qual_scores);
+        m_tmp_sequence += read.Sequence();
+        m_read_fingerprint.record( read.Sequence() );
+        read.GetQualScores(m_qual_scores);
         read_start[read_num] = start;
         auto sz = read.Sequence().size();
         start += sz;
@@ -429,6 +576,19 @@ void fastq_writer_vdb::write_spot(const vector<CFastqRead>& reads)
         //SRA_READ_TYPE_TECHNICAL = 0;
         //SRA_READ_TYPE_BIOLOGICAL = 1;
         read_type[read_num] = read.Type();
+/*
+        switch (mReadTypes[read_num]) {
+        case 'T':
+            read_type[read_num] = SRA_READ_TYPE_TECHNICAL;
+            break;
+        case 'B':
+            read_type[read_num] = SRA_READ_TYPE_BIOLOGICAL;
+            break;
+        default:
+            read_type[read_num] = sz < 40 ? SRA_READ_TYPE_TECHNICAL: SRA_READ_TYPE_BIOLOGICAL;
+            break;
+        }
+*/
         read_filter[read_num] = (char)read.ReadFilter();
         if ( m_platform == SRA_PLATFORM_OXFORD_NANOPORE )
         {
@@ -437,10 +597,9 @@ void fastq_writer_vdb::write_spot(const vector<CFastqRead>& reads)
         }
         ++read_num;
     }
-    std::transform(sequence.begin(), sequence.end(), sequence.begin(), ::toupper);
-    c_READ.setValue(sequence);
-    c_QUALITY.setValue(qual_scores.size(), sizeof(uint8_t), &qual_scores[0]);
-    c_READ_START.setValue(read_num, sizeof(int32_t),&read_start);
+    c_READ.setValue(m_tmp_sequence);
+    c_QUALITY.setValue(m_qual_scores.size(), sizeof(uint8_t), &m_qual_scores[0]);
+    c_READ_START.setValue(read_num, sizeof(int32_t), read_start);
     c_READ_LEN.setValue(read_num, sizeof(int32_t), read_len);
     c_READ_TYPE.setValue(read_num, sizeof(char), read_type);
     c_READ_FILTER.setValue(read_num, sizeof(char), read_filter);
@@ -451,7 +610,88 @@ void fastq_writer_vdb::write_spot(const vector<CFastqRead>& reads)
     }
     SEQUENCE_TABLE.closeRow();
 }
+using json = nlohmann::json;
+
+class fastq_writer_exp : public fastq_writer_vdb
+{
+public:
+    fastq_writer_exp(const json& ExperimentSpecs, ostream& stream, shared_ptr<Writer2> writer = shared_ptr<Writer2>())
+        : fastq_writer_vdb(stream, writer)
+    {
+            auto j = ExperimentSpecs["EXPERIMENT"]["DESIGN"]["SPOT_DESCRIPTOR"]["SPOT_DECODE_SPEC"]["READ_SPEC"];
+            if (j.is_null())
+                throw fastq_error("Experiment does not contains READ_SPEC");
+
+            if (j.is_array()) {
+                for (auto it = j.begin();it != j.end(); ++it) {
+                    auto& v = *it;
+                    auto base_coord = v["BASE_COORD"].get<string>();
+                    read_start.push_back(stoi(base_coord) - 1);
+                    auto read_class = v["READ_CLASS"].get<string>();
+                    m_read_type.push_back((read_class == "Technical Read") ? 0 : 1);
+                }
+            } else {
+                auto base_coord = j["BASE_COORD"].get<string>();
+                read_start.push_back(stoi(base_coord) - 1);
+                auto read_class = j["READ_CLASS"].get<string>();
+                m_read_type.push_back((read_class == "Technical Read") ? 0 : 1);
+            }
+
+        auto num_reads = read_start.size();
+        read_len.resize(num_reads, 0);
+        read_filter.resize(num_reads, 0);
+    }
 
 
+    void write_spot(const string& spot_name, const vector<CFastqRead>& reads) override
+    {
+        if (reads.empty())
+            return;
+        //auto table = m_writer->table("SEQUENCE");
+        const auto& first_read = reads.front();
+        //m_tmp_spot = first_read.Spot();
+        m_tmp_spot = spot_name;
+        m_tmp_spot += first_read.Suffix();
+        c_NAME.setValue(m_tmp_spot);
+        c_SPOT_GROUP.setValue(first_read.SpotGroup());
+        c_PLATFORM.setValue(m_platform);
+        m_tmp_sequence = first_read.Sequence();
+        m_qual_scores.clear();
+        first_read.GetQualScores(m_qual_scores);
+        auto read_num = read_start.size();
+        auto sz = m_tmp_sequence.size();
+        for (int i = read_num - 1; i >=0; --i) {
+            read_len[i] = sz - read_start[i];
+            sz -= read_len[i];
+        }
+        c_READ.setValue(m_tmp_sequence);
+        c_QUALITY.setValue(m_qual_scores.size(), sizeof(uint8_t), &m_qual_scores[0]);
+        c_READ_START.setValue(read_num, sizeof(int32_t),read_start.data());
+        c_READ_LEN.setValue(read_num, sizeof(int32_t), read_len.data());
+        c_READ_TYPE.setValue(read_num, sizeof(char), mReadTypes.data());//        m_read_type.data());
+        c_READ_FILTER.setValue(read_num, sizeof(char), read_filter.data());
+        SEQUENCE_TABLE.closeRow();
+    }
+protected:
+    vector<int32_t> read_start;
+    vector<int32_t> read_len;
+    vector<char> m_read_type;
+    vector<char> read_filter;
+
+
+};
+
+template<typename Mutex>
+void general_writer_sink<Mutex>::sink_it_(const spdlog::details::log_msg& msg)
+{
+    spdlog::memory_buf_t formatted;
+    spdlog::sinks::base_sink<Mutex>::formatter_->format(msg, formatted);
+    m_writer->add_message(msg.level, fmt::to_string(formatted));
+    /*
+    if (msg.level >= SPDLOG_LEVEL_ERROR)
+        writer->errorMessage(fmt::to_string(formatted));
+    else
+        writer->logMessage(fmt::to_string(formatted));
+    */
+}
 
-#endif
