@@ -9,8 +9,8 @@
  *  @license GPL3 <https://www.gnu.org/licenses/gpl-3.0-standalone.html>
  */
 
-#ifndef CHOCOBO1_BLAKE1_384_H
-#define CHOCOBO1_BLAKE1_384_H
+#ifndef CHOCOBO1_BLAKE1_512_H
+#define CHOCOBO1_BLAKE1_512_H
 
 #include <array>
 #include <cassert>
@@ -44,7 +44,7 @@
 namespace Chocobo1
 {
 	// Use these!!
-	// Blake1_384();
+	// Blake1_512();
 }
 
 
@@ -227,15 +227,15 @@ namespace Hash
 #endif
 
 
-namespace Blake1_384_NS
+namespace Blake1_512_NS
 {
-	class Blake1_384
+	class Blake1_512
 	{
 		// https://131002.net/blake/
 
 		public:
 			using Byte = uint8_t;
-			using ResultArrayType = std::array<Byte, 48>;
+			using ResultArrayType = std::array<Byte, 64>;
 
 #if (USE_STD_SPAN_CHOCOBO1_HASH == 1)
 			template <typename T, std::size_t Extent = std::dynamic_extent>
@@ -246,23 +246,23 @@ namespace Blake1_384_NS
 #endif
 
 
-			constexpr Blake1_384();
+			constexpr Blake1_512();
 
 			constexpr void reset();
-			CONSTEXPR_CPP17_CHOCOBO1_HASH Blake1_384& finalize();  // after this, only `toArray()`, `toString()`, `toVector()`, `reset()` are available
+			CONSTEXPR_CPP17_CHOCOBO1_HASH Blake1_512& finalize();  // after this, only `toArray()`, `toString()`, `toVector()`, `reset()` are available
 
 			std::string toString() const;
 			std::vector<Byte> toVector() const;
 			CONSTEXPR_CPP17_CHOCOBO1_HASH ResultArrayType toArray() const;
 
-			constexpr Blake1_384& addData(const Span<const Byte> inData);
-			constexpr Blake1_384& addData(const void *ptr, const std::size_t length);
+			constexpr Blake1_512& addData(const Span<const Byte> inData);
+			constexpr Blake1_512& addData(const void *ptr, const std::size_t length);
 			template <std::size_t N>
-			constexpr Blake1_384& addData(const Byte (&array)[N]);
+			constexpr Blake1_512& addData(const Byte (&array)[N]);
 			template <typename T, std::size_t N>
-			Blake1_384& addData(const T (&array)[N]);
+			Blake1_512& addData(const T (&array)[N]);
 			template <typename T>
-			Blake1_384& addData(const Span<T> inSpan);
+			Blake1_512& addData(const Span<T> inSpan);
 
 		private:
 			constexpr void addDataImpl(const Span<const Byte> data, const int paddingLen = 0);
@@ -281,7 +281,7 @@ namespace Blake1_384_NS
 			};
 	};
 
-	constexpr uint64_t Blake1_384::cTable[16];
+	constexpr uint64_t Blake1_512::cTable[16];
 
 
 	// helpers
@@ -334,28 +334,28 @@ namespace Blake1_384_NS
 
 
 	//
-	constexpr Blake1_384::Blake1_384()
+	constexpr Blake1_512::Blake1_512()
 	{
 		static_assert((CHAR_BIT == 8), "Sorry, we don't support exotic CPUs");
 		reset();
 	}
 
-	constexpr void Blake1_384::reset()
+	constexpr void Blake1_512::reset()
 	{
 		m_buffer.clear();
 		m_sizeCounter = 0;
 
-		m_h[0] = 0xcbbb9d5dc1059ed8;
-		m_h[1] = 0x629a292a367cd507;
-		m_h[2] = 0x9159015a3070dd17;
-		m_h[3] = 0x152fecd8f70e5939;
-		m_h[4] = 0x67332667ffc00b31;
-		m_h[5] = 0x8eb44a8768581511;
-		m_h[6] = 0xdb0c2e0d64f98fa7;
-		m_h[7] = 0x47b5481dbefa4fa4;
+		m_h[0] = 0x6a09e667f3bcc908;
+		m_h[1] = 0xbb67ae8584caa73b;
+		m_h[2] = 0x3c6ef372fe94f82b;
+		m_h[3] = 0xa54ff53a5f1d36f1;
+		m_h[4] = 0x510e527fade682d1;
+		m_h[5] = 0x9b05688c2b3e6c1f;
+		m_h[6] = 0x1f83d9abfb41bd6b;
+		m_h[7] = 0x5be0cd19137e2179;
 	}
 
-	CONSTEXPR_CPP17_CHOCOBO1_HASH Blake1_384& Blake1_384::finalize()
+	CONSTEXPR_CPP17_CHOCOBO1_HASH Blake1_512& Blake1_512::finalize()
 	{
 		const Uint128 sizeCounterBits = (m_sizeCounter + (m_buffer.size() * 8));
 		const uint64_t sizeCounterBitsL = sizeCounterBits.low();
@@ -368,6 +368,8 @@ namespace Blake1_384_NS
 		const auto len = static_cast<int>(((2 * BLOCK_SIZE) - (m_buffer.size() + 16)) % BLOCK_SIZE);
 		m_buffer.fill(0, (len + 16));
 
+		m_buffer[m_buffer.size() - 17] |= 1;
+
 		// append size in bits
 		for (int i = 0; i < 8; ++i)
 		{
@@ -381,7 +383,7 @@ namespace Blake1_384_NS
 		return (*this);
 	}
 
-	std::string Blake1_384::toString() const
+	std::string Blake1_512::toString() const
 	{
 		const auto a = toArray();
 		std::string ret;
@@ -400,15 +402,15 @@ namespace Blake1_384_NS
 		return ret;
 	}
 
-	std::vector<Blake1_384::Byte> Blake1_384::toVector() const
+	std::vector<Blake1_512::Byte> Blake1_512::toVector() const
 	{
 		const auto a = toArray();
 		return {a.begin(), a.end()};
 	}
 
-	CONSTEXPR_CPP17_CHOCOBO1_HASH Blake1_384::ResultArrayType Blake1_384::toArray() const
+	CONSTEXPR_CPP17_CHOCOBO1_HASH Blake1_512::ResultArrayType Blake1_512::toArray() const
 	{
-		const Span<const uint64_t> state(std::begin(m_h), (std::end(m_h) - 2));
+		const Span<const uint64_t> state(m_h);
 		const int dataSize = sizeof(decltype(state)::value_type);
 
 		ResultArrayType ret {};
@@ -422,7 +424,7 @@ namespace Blake1_384_NS
 		return ret;
 	}
 
-	constexpr Blake1_384& Blake1_384::addData(const Span<const Byte> inData)
+	constexpr Blake1_512& Blake1_512::addData(const Span<const Byte> inData)
 	{
 		Span<const Byte> data = inData;
 
@@ -456,31 +458,31 @@ namespace Blake1_384_NS
 		return (*this);
 	}
 
-	constexpr Blake1_384& Blake1_384::addData(const void *ptr, const std::size_t length)
+	constexpr Blake1_512& Blake1_512::addData(const void *ptr, const std::size_t length)
 	{
 		// Span::size_type = std::size_t
 		return addData({static_cast<const Byte*>(ptr), length});
 	}
 
 	template <std::size_t N>
-	constexpr Blake1_384& Blake1_384::addData(const Byte (&array)[N])
+	constexpr Blake1_512& Blake1_512::addData(const Byte (&array)[N])
 	{
 		return addData({array, N});
 	}
 
 	template <typename T, std::size_t N>
-	Blake1_384& Blake1_384::addData(const T (&array)[N])
+	Blake1_512& Blake1_512::addData(const T (&array)[N])
 	{
 		return addData({reinterpret_cast<const Byte*>(array), (sizeof(T) * N)});
 	}
 
 	template <typename T>
-	Blake1_384& Blake1_384::addData(const Span<T> inSpan)
+	Blake1_512& Blake1_512::addData(const Span<T> inSpan)
 	{
 		return addData({reinterpret_cast<const Byte*>(inSpan.data()), inSpan.size_bytes()});
 	}
 
-	constexpr void Blake1_384::addDataImpl(const Span<const Byte> data, const int paddingLen)
+	constexpr void Blake1_512::addDataImpl(const Span<const Byte> data, const int paddingLen)
 	{
 		assert((data.size() % BLOCK_SIZE) == 0);
 
@@ -519,7 +521,7 @@ namespace Blake1_384_NS
 				a = (a + b + (y ^ constant2)); \
 				d = rotr((d ^ a), 16); \
 				c = (c + d); \
-				b = rotr((b ^ c), 11);
+				b = rotr((b ^ c), 11); \
 
 			// had to unroll loop manually :(
 			blakeMix(v[0], v[4], v[8], v[12], m[0], m[1], cTable[1], cTable[0]);
@@ -681,7 +683,7 @@ namespace Blake1_384_NS
 	}
 }
 }
-	using Blake1_384 = Hash::Blake1_384_NS::Blake1_384;
+	using Blake1_512 = Hash::Blake1_512_NS::Blake1_512;
 }
 
-#endif  // CHOCOBO1_BLAKE1_384_H
+#endif  // CHOCOBO1_BLAKE1_512_H
