@@ -14,7 +14,7 @@
 // #define SHARQ_VERSION_PATCH 3 /// Doxygen comments; defline_matcher groupSize refectoring
 
 // #define SHARQ_VERSION to_string(SHARQ_VERSION_MAJOR) + "." + to_string(SHARQ_VERSION_MINOR) + "." + to_string(SHARQ_VERSION_PATCH)
-#include "shared/toolkit.vers.h"
+#include <shared/toolkit.vers.h>
 #define SHARQ_VERSION ( to_string(TOOLKIT_VERS >> 24) + "." + to_string((TOOLKIT_VERS >> 16) & 0x00ff) + "." + to_string(TOOLKIT_VERS & 0xffff) )
 
 #endif
